#pragma once

// CliGw includes.
#include "connection_manager.hpp"

// AFAP includes.
#include "components/worker_node/src/node.hpp"

// STL includes.
#include <shared_mutex>

namespace CliGw
{
    class Node : public workers::node,
                 public connections::interfaces::handler::server
    {
    public:
        explicit Node(const TypeInst &ownInst,
                      boost::asio::io_context &ioContext,
                      const configuration::manager &configManager);

        void start() override;
        void stop() override;

    protected:
        void notifyPrimitive(const primitive::GenericPtr &prim) override;
        void handleSpecificPrimitive(const primitive::GenericPtr &prim) override;

        void handleRegisterCmdInd(const primitive::GenericPtr &prim);

        void handleExecCmdReq(const primitive::GenericPtr &prim);
        void handleExecCmdRsp(const primitive::GenericPtr &prim);

    private:
        std::shared_ptr<connections::manager> m_connectionManager;

        std::shared_mutex m_cmdsMapMutex;
        std::map<std::string, TypeInst> m_cmdsMap;
    };

    using API = workers::api<Node, workers::type::CLIGW, 1>;

} // namespace CliGw
