#include "asn1_tools.hpp"

namespace asn1
{

    RequestParams::RequestParams(uint8_t *buf_p1, size_t size_p1,
                                 uint8_t *buf_p2, size_t size_p2,
                                 uint8_t *buf_p3, size_t size_p3,
                                 uint8_t *buf_p4, size_t size_p4)
    {
        memset(this, 0, sizeof(RequestParms_t));

        if ((buf_p1 != nullptr) && (size_p1 != 0))
        {
            sessionKey = new SessionKey_t();
            sessionKey->buf = new uint8_t[size_p1];
            sessionKey->size = size_p1;
            memcpy(sessionKey->buf, buf_p1, sessionKey->size);
        }

        callingPartyNumber.buf = new uint8_t[size_p2];
        callingPartyNumber.size = size_p2;
        memcpy(callingPartyNumber.buf, buf_p2, callingPartyNumber.size);

        calledPartyNumber.buf = new uint8_t[size_p3];
        calledPartyNumber.size = size_p3;
        memcpy(calledPartyNumber.buf, buf_p3, calledPartyNumber.size);

        if ((buf_p4 != nullptr) && (size_p4 != 0))
        {
            originalCalledNumber = new OriginalCalledNumber_t();
            originalCalledNumber->buf = new uint8_t[size_p4];
            originalCalledNumber->size = size_p4;
            memcpy(originalCalledNumber->buf, buf_p4, originalCalledNumber->size);
        }
    }

    RequestParams::~RequestParams()
    {
        if (sessionKey != nullptr)
        {
            delete[] sessionKey->buf;
            delete sessionKey;
        }
        delete[] callingPartyNumber.buf;
        delete[] calledPartyNumber.buf;
        if (originalCalledNumber != nullptr)
        {
            delete[] originalCalledNumber->buf;
            delete originalCalledNumber;
        }
    }

    asn_enc_rval_t RequestParams::AsnEncode(void *buffer, size_t buffer_size)
    {
        return asn_encode_to_buffer(0, ATS_DER, &asn_DEF_RequestParms, this, buffer, buffer_size);
    }

    RequestParamsPtr RequestParams::AsnDecode(const void *buffer, size_t size, asn_dec_rval_t *_rval)
    {
        asn_dec_rval_t rval;
        RequestParms_t *req = nullptr;
        rval = asn_decode(0, ATS_BER, &asn_DEF_RequestParms, (void **)&req, buffer, size);
        if (_rval != nullptr)
        {
            *_rval = rval;
        }
        if (rval.code == RC_OK)
        {
            return RequestParamsPtr(req, [](RequestParms_t *r)
                                    { ASN_STRUCT_FREE(asn_DEF_RequestParms, r); });
        }
        else
        {
            return RequestParamsPtr();
        }
    }

    ContinueParams::ContinueParams(uint8_t *buf_p1, size_t size_p1)
    {
        memset(this, 0, sizeof(ContinueParms_t));

        if ((buf_p1 != nullptr) && (size_p1 != 0))
        {
            sessionKey = new SessionKey_t();
            sessionKey->buf = new uint8_t[size_p1];
            sessionKey->size = size_p1;
            memcpy(sessionKey->buf, buf_p1, sessionKey->size);
        }
    }

    ContinueParams::~ContinueParams()
    {
        if (sessionKey != nullptr)
        {
            delete[] sessionKey->buf;
            delete sessionKey;
        }
    }

    asn_enc_rval_t ContinueParams::AsnEncode(void *buffer, size_t buffer_size)
    {
        return asn_encode_to_buffer(0, ATS_DER, &asn_DEF_ContinueParms, this, buffer, buffer_size);
    }

    ContinueParamsPtr ContinueParams::AsnDecode(const void *buffer, size_t size, asn_dec_rval_t *_rval)
    {
        asn_dec_rval_t rval;
        ContinueParms_t *req = nullptr;
        rval = asn_decode(0, ATS_BER, &asn_DEF_ContinueParms, (void **)&req, buffer, size);
        if (_rval != nullptr)
        {
            *_rval = rval;
        }
        if (rval.code == RC_OK)
        {
            return ContinueParamsPtr(req, [](ContinueParms_t *r)
                                     { ASN_STRUCT_FREE(asn_DEF_ContinueParms, r); });
        }
        else
        {
            return ContinueParamsPtr();
        }
    }

    ReleaseParams::ReleaseParams(uint8_t *buf_p1, size_t size_p1,
                                 long p2)
    {
        memset(this, 0, sizeof(ReleaseParms_t));

        if ((buf_p1 != nullptr) && (size_p1 != 0))
        {
            sessionKey = new SessionKey_t();
            sessionKey->buf = new uint8_t[size_p1];
            sessionKey->size = size_p1;
            memcpy(sessionKey->buf, buf_p1, sessionKey->size);
        }

        releaseCode = p2;
    }

    ReleaseParams::ReleaseParams(uint8_t *buf_p1, size_t size_p1,
                                 long p2, long p3)
    {
        memset(this, 0, sizeof(ReleaseParms_t));

        if ((buf_p1 != nullptr) && (size_p1 != 0))
        {
            sessionKey = new SessionKey_t();
            sessionKey->buf = new uint8_t[size_p1];
            sessionKey->size = size_p1;
            memcpy(sessionKey->buf, buf_p1, sessionKey->size);
        }

        releaseCode = p2;

        additionalCode = new long(p3);
    }

    ReleaseParams::~ReleaseParams()
    {
        if (sessionKey != nullptr)
        {
            delete[] sessionKey->buf;
            delete sessionKey;
        }
        delete additionalCode;
    }

    asn_enc_rval_t ReleaseParams::AsnEncode(void *buffer, size_t buffer_size)
    {
        return asn_encode_to_buffer(0, ATS_DER, &asn_DEF_ReleaseParms, this, buffer, buffer_size);
    }

    ReleaseParamsPtr ReleaseParams::AsnDecode(const void *buffer, size_t size, asn_dec_rval_t *_rval)
    {
        asn_dec_rval_t rval;
        ReleaseParms_t *req = nullptr;
        rval = asn_decode(0, ATS_BER, &asn_DEF_ReleaseParms, (void **)&req, buffer, size);
        if (_rval != nullptr)
        {
            *_rval = rval;
        }
        if (rval.code == RC_OK)
        {
            return ReleaseParamsPtr(req, [](ReleaseParms_t *r)
                                    { ASN_STRUCT_FREE(asn_DEF_ReleaseParms, r); });
        }
        else
        {
            return ReleaseParamsPtr();
        }
    }

    static int hexchar2int(unsigned char c)
    {
        switch (c)
        {
        case '0':
            return 0;
        case '1':
            return 1;
        case '2':
            return 2;
        case '3':
            return 3;
        case '4':
            return 4;
        case '5':
            return 5;
        case '6':
            return 6;
        case '7':
            return 7;
        case '8':
            return 8;
        case '9':
            return 9;
        case 'a':
        case 'A':
            return 0x0A;
        case 'b':
        case 'B':
            return 0x0B;
        case 'c':
        case 'C':
            return 0x0C;
        case 'd':
        case 'D':
            return 0x0D;
        case 'e':
        case 'E':
            return 0x0E;
        case 'f':
        case 'F':
            return 0x0F;
        }
        return -1;
    }

    std::string UndoText(const std::string &text)
    {
        if (text.size() % 2)
        {
            return std::string();
        }

        std::string ret(text.size() >> 1, 0);

        char *q = &(ret[0]);
        char ch, cl;
        int chi, cli;
        const char *p = (const char *)text.c_str();
        size_t cnt = 0;

        while (*p)
        {
            ch = *p++;
            cl = *p++;
            cli = hexchar2int(cl);
            chi = hexchar2int(ch);
            cnt++;
            *q++ = (char)((chi << 4) | cli);
        }

        return ret;
    }

}
