#include "components/asn1/src/RequestParms.h"
#include "components/asn1/src/ContinueParms.h"
#include "components/asn1/src/ReleaseParms.h"

#include <string>
#include <memory>

namespace asn1
{
    struct RequestParams : public RequestParms_t
    {
        RequestParams(uint8_t *buf_p1, size_t size_p1,
                      uint8_t *buf_p2, size_t size_p2,
                      uint8_t *buf_p3, size_t size_p3,
                      uint8_t *buf_p4 = nullptr, size_t size_p4 = 0);
        ~RequestParams();

        asn_enc_rval_t AsnEncode(void *buffer, size_t buffer_size);
        static std::shared_ptr<RequestParms_t> AsnDecode(const void *buffer, size_t size, asn_dec_rval_t *rval = nullptr);
    };
    using RequestParamsPtr = std::shared_ptr<RequestParms_t>;

    struct ContinueParams : public ContinueParms_t
    {
        ContinueParams(uint8_t *buf_p1, size_t size_p1);
        ~ContinueParams();

        asn_enc_rval_t AsnEncode(void *buffer, size_t buffer_size);
        static std::shared_ptr<ContinueParms_t> AsnDecode(const void *buffer, size_t size, asn_dec_rval_t *rval = nullptr);
    };
    using ContinueParamsPtr = std::shared_ptr<ContinueParms_t>;

    struct ReleaseParams : public ReleaseParms_t
    {
        ReleaseParams(uint8_t *buf_p1, size_t size_p1,
                      long p2);
        ReleaseParams(uint8_t *buf_p1, size_t size_p1,
                      long p2, long p3);
        ~ReleaseParams();

        asn_enc_rval_t AsnEncode(void *buffer, size_t buffer_size);
        static std::shared_ptr<ReleaseParms_t> AsnDecode(const void *buffer, size_t size, asn_dec_rval_t *rval = nullptr);
    };
    using ReleaseParamsPtr = std::shared_ptr<ReleaseParms_t>;

    std::string UndoText(const std::string &text);

}
