#pragma once

// AFAP includes.
#include "components/db/src/types.hpp"
#include "components/support/interfaces_template.hpp"

namespace db
{
    namespace interfaces
    {
        INTERFACE_S(notifier,
                    INTERFACE_SERVER(void, notify_row_added, const table::name &, void *);
                    INTERFACE_SERVER(void, notify_row_updated, const table::name &, void *);
                    INTERFACE_SERVER(void, notify_row_deleted, const table::name &, void *);)

        INTERFACE_C(notifier,
                    INTERFACE_CLIENT(void, notify_row_added, const table::name &, void *);
                    INTERFACE_CLIENT(void, notify_row_updated, const table::name &, void *);
                    INTERFACE_CLIENT(void, notify_row_deleted, const table::name &, void *);)

        INTERFACE_S(tables,
                    INTERFACE_SERVER(void, create, const table::name &, const table::fields &, const table::settings &);
                    INTERFACE_SERVER(void, drop, const table::name &);
                    INTERFACE_SERVER(void, recreate, const table::name &, const table::fields &, const table::settings &);
                    INTERFACE_SERVER(void, fill_in, const table::name &, const std::string &);
                    INTERFACE_SERVER(void, subscribe_on, const table::name &);
                    INTERFACE_SERVER(limit_t, size, const table::name &);
                    INTERFACE_SERVER(bool, is_exist, const table::name &);)
        INTERFACE_C(tables,
                    INTERFACE_CLIENT(void, create, const table::name &, const table::fields &, const table::settings &);
                    INTERFACE_CLIENT(void, drop, const table::name &);
                    INTERFACE_CLIENT(void, recreate, const table::name &, const table::fields &, const table::settings &);
                    INTERFACE_CLIENT(void, fill_in, const table::name &, const std::string &);
                    INTERFACE_CLIENT(void, subscribe_on, const table::name &);
                    INTERFACE_CLIENT_R(limit_t, size, const table::name &);
                    INTERFACE_CLIENT_R(bool, is_exist, const table::name &);)

        namespace tables
        {
            INTERFACE_S(calls,
                        INTERFACE_SERVER(void, put, const Uvr::CallRecord &);
                        INTERFACE_SERVER(bool, update, const Uvr::CallRecord &);
                        INTERFACE_SERVER(void, get,
                                         Uvr::CallInformationResponses &,
                                         const Uvr::CallInformationRequest &);
                        INTERFACE_PROPERTY(db::table::name, tableName);
                        INTERFACE_PROPERTY(db::table::fields, tableFields);
                        INTERFACE_PROPERTY(db::table::settings, tableSettings);
                        INTERFACE_PROPERTY(db::table::selectSettings, selectSettings);)

            INTERFACE_C(calls,
                        INTERFACE_CLIENT(void, put, const Uvr::CallRecord &);
                        INTERFACE_CLIENT_R(bool, update, const Uvr::CallRecord &);
                        INTERFACE_CLIENT(void, get,
                                         Uvr::CallInformationResponses &,
                                         const Uvr::CallInformationRequest &);)

            INTERFACE_S(incidents,
                        INTERFACE_SERVER(void, get,
                                         incident::rows &,
                                         const ::timers::time_point &,
                                         const ::timers::time_point &);
                        INTERFACE_SERVER(void, put, const incident::record &);
                        INTERFACE_PROPERTY(db::table::name, tableName);
                        INTERFACE_PROPERTY(db::table::fields, tableFields);
                        INTERFACE_PROPERTY(db::table::settings, tableSettings);)
            INTERFACE_C(incidents,
                        INTERFACE_CLIENT(void, get,
                                         incident::rows &,
                                         const ::timers::time_point &,
                                         const ::timers::time_point &);
                        INTERFACE_CLIENT(void, put, const incident::record &);)

            INTERFACE_S(incidentsA,
                        INTERFACE_SERVER(void, get,
                                         incidentA::rows &,
                                         const ::timers::time_point &,
                                         const ::timers::time_point &);
                        INTERFACE_SERVER(void, put, const incidentA::record &);
                        INTERFACE_PROPERTY(db::table::name, tableName);
                        INTERFACE_PROPERTY(db::table::fields, tableFields);
                        INTERFACE_PROPERTY(db::table::settings, tableSettings);)
            INTERFACE_C(incidentsA,
                        INTERFACE_CLIENT(void, get,
                                         incidentA::rows &,
                                         const ::timers::time_point &,
                                         const ::timers::time_point &);
                        INTERFACE_CLIENT(void, put, const incidentA::record &);)

            INTERFACE_S(subscribers,
                        INTERFACE_SERVER(void, populate, const subscriber::rows &);
                        INTERFACE_SERVER(void, getall, const std::function<void(db::subscriber::record &&)> &);
                        INTERFACE_SERVER(void, get,
                                         subscriber::rows &, limit_t, limit_t);
                        INTERFACE_SERVER(void, put, const subscriber::record &);
                        INTERFACE_SERVER(void, update, const subscriber::record &);
                        INTERFACE_SERVER(void, remove, const Uvr::Number &);
                        INTERFACE_PROPERTY(db::table::name, tableName);
                        INTERFACE_PROPERTY(db::table::fields, tableFields);
                        INTERFACE_PROPERTY(db::table::settings, tableSettings);)
            INTERFACE_C(subscribers,
                        INTERFACE_CLIENT(void, populate, const subscriber::rows &);
                        INTERFACE_CLIENT(void, getall, const std::function<void(db::subscriber::record &&)> &);
                        INTERFACE_CLIENT(void, get,
                                         subscriber::rows &, limit_t, limit_t);
                        INTERFACE_CLIENT(void, put, const subscriber::record &);
                        INTERFACE_CLIENT(void, update, const subscriber::record &);
                        INTERFACE_CLIENT(void, remove, const Uvr::Number &);)

            INTERFACE_S(uvrs,
                        INTERFACE_SERVER(void, populate, const uvr::rows &);
                        INTERFACE_SERVER(void, put, const uvr::record &);
                        INTERFACE_SERVER(bool, get, uvr::record &, const Uvr::UvrId_t &);
                        INTERFACE_SERVER(void, get, uvr::rows &, const Uvr::Ip_t &);
                        INTERFACE_PROPERTY(db::table::name, tableName);
                        INTERFACE_PROPERTY(db::table::fields, tableFields);
                        INTERFACE_PROPERTY(db::table::settings, tableSettings);)
            INTERFACE_C(uvrs,
                        INTERFACE_CLIENT(void, populate, const uvr::rows &);
                        INTERFACE_CLIENT(void, put, const uvr::record &);
                        INTERFACE_CLIENT_R(bool, get, uvr::record &, const Uvr::UvrId_t &);
                        INTERFACE_CLIENT(void, get, uvr::rows &, const Uvr::Ip_t &);)

            INTERFACE_S(hubs,
                        INTERFACE_SERVER(void, populate, const hub::rows &);
                        INTERFACE_SERVER(void, get, hub::rows &, const gts_t &);
                        INTERFACE_SERVER(void, put, const hub::record &);
                        INTERFACE_PROPERTY(db::table::name, tableName);
                        INTERFACE_PROPERTY(db::table::fields, tableFields);
                        INTERFACE_PROPERTY(db::table::settings, tableSettings);)
            INTERFACE_C(hubs,
                        INTERFACE_CLIENT(void, populate, const hub::rows &);
                        INTERFACE_CLIENT(void, get, hub::rows &, const gts_t &);
                        INTERFACE_CLIENT(void, put, const hub::record &);)

            INTERFACE_S(Operators,
                        INTERFACE_SERVER(void, populate, const Operator::Rows &);
                        INTERFACE_SERVER(void, get, Operator::Rows &, const OperatorsIds &);
                        INTERFACE_SERVER(void, put, const Operator::Record &);
                        INTERFACE_PROPERTY(db::table::name, tableName);
                        INTERFACE_PROPERTY(db::table::fields, tableFields);
                        INTERFACE_PROPERTY(db::table::settings, tableSettings);)
            INTERFACE_C(Operators,
                        INTERFACE_CLIENT(void, populate, const Operator::Rows &);
                        INTERFACE_CLIENT(void, get, Operator::Rows &, const OperatorsIds &);
                        INTERFACE_CLIENT(void, put, const Operator::Record &);)

        } // namespace tables

    } // namespace interfaces

} // namespace db
