#include "db/src/types.hpp"

// AFAP includes.
#include "components/csv/src/CsvFile.hpp"
#include "TRecQ763Number.hpp"

namespace db
{
    namespace incident
    {
        std::ostream &operator<<(std::ostream &os, const record &r)
        {
            std::stringbuf trace;
            std::ostream logger(&trace);

            logger << TRecQ763Number::toStdString(r.m_numA) << Csv::delimiter
                   << TRecQ763Number::toStdString(r.m_numB) << Csv::delimiter
                   << TRecQ763Number::toStdString(r.m_numD) << Csv::delimiter
                   << TRecQ763Number::toStdString(r.m_numC) << Csv::delimiter
                   << r.m_date.format() << Csv::delimiter
                   << static_cast<int>(r.m_idRel) << Csv::delimiter
                   << static_cast<int>(r.m_rlc) << Csv::delimiter
                   << r.m_idSrc << Csv::delimiter
                   << r.m_idUvrT << Csv::delimiter
                   << r.callId;
            os << trace.str();
            return os;
        }
    } // namespace incident

    namespace incidentA
    {
        std::ostream &operator<<(std::ostream &os, const record &r)
        {
            std::stringbuf trace;
            std::ostream logger(&trace);

            logger << TRecQ763Number::toStdString(r.m_numA) << Csv::delimiter
                   << TRecQ763Number::toStdString(r.m_numB) << Csv::delimiter
                   << TRecQ763Number::toStdString(r.m_numC) << Csv::delimiter
                   << r.m_date.format() << Csv::delimiter
                   << static_cast<int>(r.m_idRel) << Csv::delimiter
                   << static_cast<int>(r.m_rlc) << Csv::delimiter
                   << r.m_idUvrO;
            os << trace.str();
            return os;
        }
    } // namespace incidentA

    namespace subscriber
    {
        record::record(const char *tmp, const char delimeter)
        {
            Csv::File::OptionalValue<Uvr::UvrId_t> idUvrS = {.value = m_idUvrS,
                                                             .defaultValue = 0};
            Csv::File::OptionalValue<std::string> metaInfo = {.value = m_metaInfo,
                                                              .defaultValue = ""};
            Csv::File::parseRow(tmp,
                                delimeter,
                                m_number,
                                m_idSrc,
                                m_idUvrP,
                                idUvrS,
                                metaInfo);
        }

        record tag_invoke(boost::json::value_to_tag<record>,
                          boost::json::value const &jv)
        {
            return record{boost::json::value_to<Uvr::Number>(jv.at("number")),
                          boost::json::value_to<Uvr::OperatorId_t>(jv.at("operator")),
                          boost::json::value_to<Uvr::UvrId_t>(jv.at("primary_uvr_id")),
                          boost::json::value_to<Uvr::UvrId_t>(jv.at("secondary_uvr_id")),
                          boost::json::value_to<Uvr::MetaInfo::String>(jv.at("meta_info"))};
        }

    } // namespace subscriber

    namespace uvr
    {
        record::record(const char *line, const char delimeter)
        {
            Csv::File::OptionalValue<Uvr::Gt_t> gtUvr = {.value = m_gtUvr,
                                                         .defaultValue = ""};
            Csv::File::OptionalValue<Uvr::Ip_t> ipUvrS = {.value = m_ipUvrS,
                                                          .defaultValue = ""};
            Csv::File::OptionalValue<Uvr::Dns_t> dnsUvr = {.value = m_dnsUvr,
                                                           .defaultValue = ""};
            Csv::File::OptionalValue<Uvr::Gt_t> idHubP = {.value = m_idHubP,
                                                          .defaultValue = ""};
            Csv::File::OptionalValue<Uvr::Gt_t> idHubS = {.value = m_idHubS,
                                                          .defaultValue = ""};
            Csv::File::OptionalValue<Uvr::Gt_t> gtUvr1 = {.value = m_gtUvr1,
                                                          .defaultValue = ""};
            Csv::File::OptionalValue<Uvr::Gt_t> gtUvr2 = {.value = m_gtUvr2,
                                                          .defaultValue = ""};
            Csv::File::OptionalValue<Uvr::OperatorId_t> idSrc = {.value = m_idSrc,
                                                                 .defaultValue = Uvr::undefinedOperatorId};

            Uvr::MetaInfo::String metaInfoStr;
            Csv::File::OptionalValue<Uvr::MetaInfo::String> metaInfo = {.value = metaInfoStr,
                                                                        .defaultValue = ""};

            Csv::File::parseRow(line,
                                delimeter,
                                m_idUvr,
                                gtUvr,
                                m_ipUvrP,
                                ipUvrS,
                                dnsUvr,
                                idHubP,
                                idHubS,
                                gtUvr1,
                                gtUvr2,
                                idSrc,
                                metaInfo);

            m_metaInfo = Uvr::UvrInfo::parseMetaInfo(metaInfoStr);
        }

        bool record::operator==(const record &other) const
        {
            return (m_idUvr == other.m_idUvr
                    && m_gtUvr == other.m_gtUvr
                    && m_ipUvrP == other.m_ipUvrP
                    && m_ipUvrS == other.m_ipUvrS
                    && m_dnsUvr == other.m_dnsUvr
                    && m_idHubP == other.m_idHubP
                    && m_idHubS == other.m_idHubS
                    && m_gtUvr1 == other.m_gtUvr1
                    && m_gtUvr2 == other.m_gtUvr2
                    && m_idSrc == other.m_idSrc
                    && m_metaInfo == other.m_metaInfo);
        }

        std::ostream &operator<<(std::ostream &os, const record &r)
        {
            std::stringbuf trace;
            std::ostream logger(&trace);

            logger << r.m_idUvr << ";"
                   << r.m_gtUvr << ";"
                   << r.m_ipUvrP << ";"
                   << r.m_ipUvrS << ";"
                   << r.m_dnsUvr << ";"
                   << r.m_idHubP << ";"
                   << r.m_idHubS << ";"
                   << r.m_gtUvr1 << ";"
                   << r.m_gtUvr2 << ";"
                   << r.m_idSrc << ";"
                   << r.m_metaInfo;

            os << trace.str();
            return os;
        }

    } // namespace uvr

    hub::record::record(const char *line, const char delimeter)
    {
        Csv::File::parseRow(line,
                            delimeter,
                            m_idHub,
                            m_ipHubP,
                            m_ipHubS,
                            m_dnsHubP,
                            m_dnsHubS);
    }

    Operator::Record::Record(const Uvr::OperatorId_t idSrc,
                             const Uvr::Name &oprName,
                             const Uvr::Name &oprNick,
                             const Uvr::Inn &inn,
                             const Uvr::OptionalBdpnCode &bdpnCode,
                             const Uvr::OptionalName &nameBrand)
        : idSrc(idSrc),
          oprName(oprName),
          oprNick(oprNick),
          inn(inn),
          bdpnCode(bdpnCode),
          nameBrand(nameBrand)
    {}

    Operator::Record::Record(const char *line, const char delimeter)
    {
        Csv::File::OptionalValue<Uvr::Inn> optionalInn = {.value = inn,
                                                          .defaultValue = ""};

        Csv::File::parseRow(line,
                            delimeter,
                            idSrc,
                            oprName,
                            oprNick,
                            optionalInn,
                            bdpnCode,
                            nameBrand);
    }

} // namespace db
