#pragma once

// AFAP includes.
#include "components/http/src/types.hpp"
#include "components/support/interfaces_template.hpp"

namespace http
{
    namespace interfaces
    {
        namespace connections
        {

            INTERFACE_S(notifier,
                        INTERFACE_SERVER(bool, connectionAccept,
                                         http::connection_ptr<role::CLIENT>);
                        INTERFACE_SERVER(bool, connectionAccept,
                                         http::connection_ptr<role::SERVER>);
                        INTERFACE_SERVER(void, connectFailure,
                                         http::connection_ptr<role::CLIENT>);
                        INTERFACE_SERVER(void, connectFailure,
                                         http::connection_ptr<role::SERVER>);
                        INTERFACE_SERVER(void, resendFailure,
                                         http::connection_ptr<role::CLIENT>);
                        INTERFACE_SERVER(void, resendFailure,
                                         http::connection_ptr<role::SERVER>);
                        INTERFACE_SERVER(void, connectionUp,
                                         http::connection_ptr<role::CLIENT>);
                        INTERFACE_SERVER(void, connectionUp,
                                         http::connection_ptr<role::SERVER>);
                        INTERFACE_SERVER(void, connectionDown,
                                         http::connections::id, role);
                        INTERFACE_SERVER(void, newMessage,
                                         const http::connection<role::SERVER> *,
                                         const http::message<messages::type::REQUEST> &);
                        INTERFACE_SERVER(void, newMessage,
                                         const http::connection<role::CLIENT> *,
                                         const http::message<messages::type::RESPONSE> &);)
            INTERFACE_C(notifier,
                        INTERFACE_CLIENT_R(bool, connectionAccept,
                                           http::connection_ptr<role::CLIENT>);
                        INTERFACE_CLIENT_R(bool, connectionAccept,
                                           http::connection_ptr<role::SERVER>);
                        INTERFACE_CLIENT(void, connectFailure,
                                         http::connection_ptr<role::CLIENT>);
                        INTERFACE_CLIENT(void, connectFailure,
                                         http::connection_ptr<role::SERVER>);
                        INTERFACE_CLIENT(void, resendFailure,
                                         http::connection_ptr<role::CLIENT>);
                        INTERFACE_CLIENT(void, resendFailure,
                                         http::connection_ptr<role::SERVER>);
                        INTERFACE_CLIENT(void, connectionUp,
                                         http::connection_ptr<role::CLIENT>);
                        INTERFACE_CLIENT(void, connectionUp,
                                         http::connection_ptr<role::SERVER>);
                        INTERFACE_CLIENT(void, connectionDown,
                                         http::connections::id, role);
                        INTERFACE_CLIENT(void, newMessage,
                                         const http::connection<role::SERVER> *,
                                         const http::message<messages::type::REQUEST> &);
                        INTERFACE_CLIENT(void, newMessage,
                                         const http::connection<role::CLIENT> *,
                                         const http::message<messages::type::RESPONSE> &);)
            INTERFACE_S(resolver,
                        INTERFACE_SERVER(void, getAddress,
                                         uint, const std::string &, std::string &);)
            INTERFACE_C(resolver,
                        INTERFACE_CLIENT(void, getAddress,
                                         uint, const std::string &, std::string &);)
        } // namespace connections

        namespace messages
        {
            INTERFACE_S(handler,
                        INTERFACE_SERVER(void, handleRequest,
                                         http::connections::id,
                                         const std::string_view &,
                                         const std::string &);
                        INTERFACE_SERVER(void, handleResponse,
                                         http::messages::result,
                                         const http::connection<http::role::CLIENT> *,
                                         const std::string &);)
            INTERFACE_C(handler,
                        INTERFACE_CLIENT(void, handleRequest,
                                         http::connections::id,
                                         const std::string_view &,
                                         const std::string &);
                        INTERFACE_CLIENT(void, handleResponse,
                                         http::messages::result,
                                         const http::connection<http::role::CLIENT> *,
                                         const std::string &);)

        } // namespace messages

    } // namespace interfaces

} // namespace http
