#pragma once

// AFAP includes.
#include "components/support/identifier.hpp"

// Boost includes.
#include <boost/beast/http.hpp>

namespace http
{
    using string_body = boost::beast::http::string_body;
    template <bool isClient>
    using message = boost::beast::http::message<isClient, string_body>;

    template <bool isClient>
    class connection;
    template <bool isClient>
    using connection_ptr = std::shared_ptr<connection<isClient>>;

    enum role
    {
        SERVER = 0,
        CLIENT = 1
    };

    namespace connections
    {
        using id = identifier;

        // Set TCP send/receive buffers up to 6 MB
        // To apply such settings, need to update the kernel parameters below
        // by the following commands:
        //  echo 'net.core.wmem_max=12582912' >> /etc/sysctl.conf
        //  echo 'net.core.rmem_max=12582912' >> /etc/sysctl.conf
        //  echo 'net.ipv4.tcp_rmem= 10240 87380 12582912' >> /etc/sysctl.conf
        //  echo 'net.ipv4.tcp_wmem= 10240 87380 12582912' >> /etc/sysctl.conf
        //  echo 'net.ipv4.tcp_window_scaling = 1' >> /etc/sysctl.conf
        //  echo 'net.ipv4.tcp_timestamps = 1' >> /etc/sysctl.conf
        //  echo 'net.ipv4.tcp_sack = 1' >> /etc/sysctl.conf
        //  echo 'net.ipv4.tcp_no_metrics_save = 1' >> /etc/sysctl.conf
        //  echo 'net.core.netdev_max_backlog = 5000' >> /etc/sysctl.conf
        //  sysctl -p

        const int tcp_buf_size = 6 * 1024 * 1024;

        enum class state
        {
            NONE,
            INIT,
            STARTED,
            DOWN,
            AUTHENTICATING,
            ESTBLISHED
        };

    } // namespace connections

    namespace messages
    {
        enum type
        {
            RESPONSE = 0,
            REQUEST = 1
        };

        using result = boost::beast::http::status;

        const boost::beast::string_view content_text = "text/plain";
        const boost::beast::string_view content_json = "application/json";

    } // namespace messages

} // namespace http
