#pragma once

// STL includes.
#include <ostream>
#include <string>

namespace Log
{
    struct SourceLocation final
    {
        SourceLocation() = default;
        SourceLocation(const char *filePath, const uint32_t lineNumber);

        bool isEmpty() const;

        friend std::ostream & operator<<(std::ostream &oStream,
                                         const SourceLocation &sourceLocation);

        std::string filePath = "";
        uint32_t lineNumber = 0;

    };
} // namespace Log

//------------------------------------------------------------------------------
// Definitions
//------------------------------------------------------------------------------

namespace Log
{
    inline SourceLocation::SourceLocation(const char *filePath,
                                          const uint32_t lineNumber)
        : filePath(filePath),
          lineNumber(lineNumber)
    {
    }

    inline bool SourceLocation::isEmpty() const
    {
        return lineNumber == 0;
    }

    inline std::ostream & operator<<(std::ostream &oStream,
                                     const SourceLocation &sourceLocation)
    {
        oStream << sourceLocation.filePath << ':' << sourceLocation.lineNumber;
        return oStream;
    }
} // namespace Log
