#pragma once

// AFAP includes.
#include "components/types/src/UvrTypes.hpp"
#include "TRecQ763Number.hpp"

namespace TRecQ763Number
{
    namespace Validator
    {
        constexpr bool emptyAllowed = true;

        /// The size to which too long numbers will be trimmed.
        constexpr size_t hashedNumberTrimSize = TRecQ763Number::hashedNumberSize + 1;
        constexpr size_t notHashedNumberTrimSize = TRecQ763Number::notHashedNumberSize + 1;

        /// @brief Verifies that the number is a valid Russian number in TRecQ763 format.
        /// @details A number is considered valid if it satisfies the following conditions:
        ///          - Hashed number:
        ///            - Length must be 10 bytes.
        ///          - Not hashed number:
        ///            - Length must be 8 bytes.
        ///            - Number must be 11 digits long.
        ///            - Number starts with the "7".
        ///            - The area code is one of the 3XX, 4XX, 8XX or 9XX.
        /// @param[in] number The number to validate.
        /// @param[in] isEmptyValid If set to @c true, then nullptr or empty @a number,
        ///                         will be considered valid. @c false by default.
        /// @param[in] trimLongNumber If set to @c true and the @a number is too long,
        ///                           it will be trimmed to a valid long + one extra byte.
        ///                           @c true by default.
        /// @returns @ref Result of validation.
        bool verify(const Uvr::OctetStringPtr &number,
                    const bool isEmptyValid = false,
                    const bool trimLongNumber = true);
    };
} // namespace TRecQ763Number
