#pragma once

// AFAP includes.
#include "components/config/src/config_manager.hpp"
#include "components/worker_node/src/typeInst.hpp"
#include "components/filesystem/src/filesystem.hpp"

namespace sftp
{
    class client
    {
        TypeInst m_ownerId;
        std::string m_sftpAuth;

    public:
        client(const TypeInst &ownerId,
               const std::string &user,
               const std::string &address,
               const std::string &port,
               const std::string &privateKey,
               const std::string &extraOptions);

        /// @brief Uploads the local file to the remote directory via SFTP
        /// @param[in] fileOrMask The specified full path of local file with filename pattern
        /// @param[in] dstDir The specified destination remote directory
        /// @return @c true if file was successfully uploaded, @c false otherwise
        bool sendFile(const std::string &fileOrMask, const std::string &dstDir) const;

        /// @brief Download the remote file to the local directory via SFTP
        /// @param[in] fileOrMask The specified full path of remote file with filename pattern
        /// @param[in] dstDir The specified destination local directory
        /// @return @c true if file was successfully downloaded, @c false otherwise
        bool getFile(const std::string &fileOrMask, const std::string &dstDir) const;

        /// @brief Returns the list of files in alphabetic order
        ///        located into the specified remote directory
        ///        and matched with the specified filename pattern
        /// @param[in] dirOrMask The specified remote directory path with filename pattern
        /// @param[out] files The list of filenames located into the specified remote directory
        ///                   and matched with the specified filename pattern
        /// @return @c true if the remote directory was listed, otherwise @c false
        bool listDir(const std::string &dirOrMask, fs::files::list &files) const;

        /// @brief Returns the latest file according to alphabetic order
        ///        among files located in the specified remote directory
        ///        and matched with the specified filename pattern
        /// @param[in] dirOrMask The specified remote directory path with filename pattern
        /// @param[out] filename The name of the latest file
        void getLatestFile(const std::string &dirOrMask, std::string &filename) const;

        /// @brief Returns @c true if SFTP client was successfully initialized, @c false otherwise.
        bool isInitialized() const;
    };
}
