#include "TimerTask.hpp"

// AFAP includes.
#include "log/src/Log.hpp"

namespace timers
{
    TimerTask::TimerTask(DeadlineTimerPtr_t timer, const HandleTimer_t &handle) : m_timer(std::move(timer)),
                                                                                     m_handle(handle) {}
    
    void TimerTask::operator()(const boost::system::error_code &err) const
    {
        if (!err)
        {
            m_handle();
        }
        else
        {
            LOGE("There is a problem with timer: ", err.what());
        }
    }

    void runDelayed(boost::asio::io_service &ioc, uint32_t duration, HandleTimer_t handle)
    {
        DeadlineTimerPtr_t timer = std::make_unique<DeadlineTimer_t>(ioc, boost::posix_time::millisec(duration));
        timer->async_wait(TimerTask(std::move(timer), handle));
    }

} // namespace timers
