#pragma once

// AFAP includes.
#include "components/log/src/Log.hpp"
#include "components/support/interfaces_template.hpp"
#include "components/timers/src/timer_types.hpp"

namespace timers
{

    namespace interfaces
    {

        INTERFACE_S(manager,
                    INTERFACE_SERVER(void, manage_timeout, const timers::id &, const ::timers::time_point &);
                    INTERFACE_SERVER(void, manage_stopped, const timers::id &);)
        INTERFACE_C1(manager,
                     INTERFACE_CLIENT1(void, manage_timeout, const timers::id &, const ::timers::time_point &);
                     INTERFACE_CLIENT1(void, manage_stopped, const timers::id &);)

    } // namespace interfaces

    class PreTimer
    {
    public:
        PreTimer(const boost::asio::strand<boost::asio::io_context::executor_type> &strand,
                 timers::duration duration) : m_duration(duration),
                                              m_strand(strand)
        {
        }

    protected:
        timers::duration m_duration;
        boost::asio::strand<boost::asio::io_context::executor_type> m_strand;
    };

    class timer : public std::enable_shared_from_this<timer>,
                  public interfaces::manager::client1,
                  public PreTimer,
                  public boost::asio::steady_timer
    {
    public:
        timer(boost::asio::io_context &ioc,
              const std::string &name,
              timers::duration duration,
              id timerId);
        timer(const boost::asio::strand<boost::asio::io_context::executor_type> &strand,
              const std::string &name,
              timers::duration duration,
              id timerId);
        virtual ~timer() = default;

        virtual void handle(const boost::system::error_code &error) final;
        virtual type getType() const = 0;

        void start();
        void stop(void);

        friend std::ostream &operator<<(std::ostream &os, const timer *t);
        template <typename T>
        friend void restartTimer(timer_ptr &timerObj);

    protected:
        id m_timerId;
        std::string m_name;
        bool m_wasCancelled{false};
        ::timers::time_point m_startTime; ///< To handle timer cancel after expiration

    private:
        virtual void handle_internal(void) = 0;

        void stop_int(void);
    };

    std::ostream &operator<<(std::ostream &os, const timer *t);

    template <class T>
    void restartTimer(timer_ptr &timerObj)
    {
        // Cancel old timer
        timerObj->stop();
        // Create a new one
        auto server = timerObj->m_server;
        timerObj.reset(new T(timerObj->m_strand, timerObj->m_name, timerObj->m_duration, timerObj->m_timerId));
        timerObj->::timers::interfaces::manager::client1::bind(server);
        timerObj->start();
        LOGD("The ", timerObj.get(), " was restarted");
    }
} // namespace timers
