#pragma once

// AFAP includes.
#include "components/timers/src/timer_object.hpp"

// Boost includes.
#include <boost/bind.hpp>

namespace timers
{
    class periodic : public timer
    {
    public:
        periodic(boost::asio::io_context &ioc,
                 const std::string &name,
                 timers::duration duration,
                 const id &timerId = id()) : timer(ioc, name, duration, timerId)
        {
        }
        periodic(const boost::asio::strand<boost::asio::io_context::executor_type> &strand,
                 const std::string &name,
                 timers::duration duration,
                 const id &timerId = id()) : timer(strand, name, duration, timerId)
        {
        }
        ~periodic() override = default;

        type getType() const override
        {
            return timers::type::PERIODIC;
        }

    protected:
        void handle_internal(void) final
        {
            expires_at(expiry() + m_duration);
            ::timers::time_point start_time;
            ::timers::now(start_time);
            async_wait(boost::bind(&timers::timer::handle,
                                   shared_from_this(),
                                   boost::asio::placeholders::error));
            interfaces::manager::client1::manage_timeout(m_timerId, m_startTime);
            m_startTime = start_time;
        }
    };
}
