#pragma once

// AFAP includes.
#include "UvrTypes.hpp"

// Boost includes.
#include "boost/json.hpp"
#include <boost/asio.hpp>

// STL includes.
#include <string>

#include <ifaddrs.h>

#define ENV(N) std::string(getenv(#N))

std::string doText(const void *buffer, size_t size);
std::string doText(const std::string &str);
std::string doText(Uvr::OctetStringPtr str);
Uvr::OctetString undoText(const std::string &md_text);

namespace console
{
    using output = std::vector<std::string>;

    output getCmdOutput(const std::string &cmd);
} // namespace console

namespace env
{
    std::string get(const std::string &key);
    std::string get(const std::string &key,
                    const std::string &defaultValue);
    size_t getSizeT(const std::string &key);
    size_t getSizeT(const std::string &key,
                    const std::string &defaultValue);
    std::optional<std::string> getOptional(const std::string &key);
} // namespace env

namespace Json
{
    enum Errors
    {
        OK = 0,
        FAIL = 1
    };

    Errors parseFile(const std::string &filename,
                     boost::json::value &fileValue);
    void prettyPrint(std::ostream &out,
                     boost::json::value const &jv,
                     std::string *indent_p = nullptr);
} // namespace Json

namespace network
{
    using address = ::boost::asio::ip::address;
    using addresses = std::vector<address>;
    namespace interface
    {
        addresses get_address(const std::string &if_name,
                              sa_family_t sa_family);
    } // namespace interface

    address make_address(const std::string &if_name,
                         sa_family_t sa_family,
                         boost::system::error_code &err);

} // namespace network

namespace io_service
{
    struct Context : public boost::asio::io_service
    {
        using work_guard = boost::asio::executor_work_guard<boost::asio::io_context::executor_type>;
        std::vector<std::shared_ptr<std::thread>> m_threads;
        std::shared_ptr<work_guard> workers_guard;

        Context() = default;
        virtual ~Context();

        void start(int threadPoolSize);
        void stop();
        void wait_stop();
    };

} // namespace io_service

namespace Digest
{

    bool shouldSendVerifyRequest(Uvr::UvrId_t uvrId);
    bool shouldIncreaseCounters(Uvr::UvrId_t uvrId);
    bool shouldMakeIncident(Uvr::UvrId_t uvrId);

} // namespace Digest
