// Own include.
#include "KeyValue.hpp"

// AFAP includes.
#include "components/log/src/Log.hpp"

// STL includes.
#include <algorithm>
#include <cstring>

namespace KeyValue
{
    PairsList parse(const std::string &str,
                    const char pairsSeparator,
                    const char keyValueSeparator)
    {
        PairsList pairsList;

        if (str.empty())
        {
            return pairsList;
        }

        const char *currentCharPtr = str.c_str();
        const char *endCharPtr = str.c_str() + str.length();
        do
        {
            // Get key.
            const char *separatorPtr = std::strchr(currentCharPtr, keyValueSeparator);
            if (separatorPtr == nullptr)
            {
                LOGERROR("Incorrect syntax of pairs list."
                         "\nExpected: \"key1=value1[,...,keyN=valueN]\""
                         "\nActual: \"",
                         str, "\"");
                pairsList.clear();
                return pairsList;
            }
            while (std::isspace(*currentCharPtr))
            {
                ++currentCharPtr;
            }
            std::string key(currentCharPtr, separatorPtr - currentCharPtr);

            // Check there is at least one character after separator.
            currentCharPtr = separatorPtr + 1;
            if (currentCharPtr >= endCharPtr)
            {
                LOGERROR("No value for key=\"", key, "\"");
                pairsList.clear();
                return pairsList;
            }

            // Get value.
            std::string value;
            separatorPtr = std::strchr(currentCharPtr, pairsSeparator);
            if (separatorPtr != nullptr)
            {
                value = std::string(currentCharPtr, separatorPtr - currentCharPtr);
                currentCharPtr = separatorPtr + 1;
            }
            else
            {
                value = std::string(currentCharPtr);
                currentCharPtr += value.length();
            }

            // Add key-value pair.
            pairsList.emplace_back(Pair(key, value));
        }
        while (currentCharPtr < endCharPtr);

        return pairsList;
    }

} // namespace KeyValue
