#pragma once

#include <boost/json.hpp>

namespace workers
{
    enum type
    {
        MANAGER,
        APIGW,
        CCNGW,
        VFC,
        VFS,
        OAM,
        VFGW,
        CLI,
        CLIGW,
        SIPMDL,
        ANY
    };

    constexpr uint8_t max_num_of_insts = 4;
    constexpr int8_t undefinedInstId = -1;
}

class TypeInst
{
    inline uint8_t encode(uint8_t type, uint8_t inst) const
    {
        return ((uint8_t)((type & 0xF) << 2) | (inst & 0x3));
    }

public:
    explicit TypeInst(uint8_t type, uint8_t inst);
    explicit TypeInst(uint8_t value);
    explicit TypeInst();

    uint8_t getType() const;
    uint8_t getInstance() const;

    uint8_t value;

    operator std::string() const;

    bool operator<(const TypeInst &a) const;

    bool operator==(const TypeInst &other) const;
    bool operator!=(const TypeInst &other) const;
};

void tag_invoke(boost::json::value_from_tag, boost::json::value &jv, TypeInst const &c);
TypeInst tag_invoke(boost::json::value_to_tag<TypeInst>, boost::json::value const &jv);

std::ostream &operator<<(std::ostream &os, const TypeInst &ti);
std::string nodeType2str(const uint8_t &type);

namespace std
{

    template <>
    struct hash<TypeInst>
    {
        std::size_t operator()(const TypeInst &typeInst) const
        {
            return typeInst.value;
        }
    };

}
