#include "pm/common/src/pm_primitive.hpp"

using namespace boost::json;

namespace pm
{

    namespace primitive
    {

        // GET_STATISTICS_REQ
        GetStatisticsReq::GetStatisticsReq(const bool reset,
                                           const counter::vectorFullIDs &counters)
            : ::primitive::generic(::primitive::GET_STATISTICS_REQ),
              reset(reset),
              counters{counters}
        {
        }

        // GET_STATISTICS_RSP
        GetStatisticsRsp::GetStatisticsRsp(const counter::container &counters,
                                           const ErrorCode &errorCode)
            : ::primitive::generic(::primitive::GET_STATISTICS_RSP),
              counters{counters},
              errorCode{errorCode}
        {
        }

        // PM_PROVIDER_REGISTER_REQ
        ProviderRegisterReq::ProviderRegisterReq(const pm::counter::vectorIDs &counters)
            : ::primitive::generic(::primitive::PM_PROVIDER_REGISTER_REQ),
              counters{counters}
        {
        }

        // GET_COUNTERS_REQ
        GetCountersReq::GetCountersReq(const bool reset,
                                       const pm::counter::vectorFullIDs &counters)
            : ::primitive::generic(::primitive::GET_COUNTERS_REQ),
              reset(reset),
              counters{counters}
        {
        }

        // GET_COUNTERS_RSP
        GetCountersRsp::GetCountersRsp(const pm::counter::container &counters)
            : ::primitive::generic(::primitive::GET_COUNTERS_RSP),
              counters{counters}
        {
        }

        void tag_invoke(boost::json::value_from_tag,
                        boost::json::value &jv,
                        const GetStatisticsRsp::ErrorCode &errorCode)
        {
            jv = static_cast<uint8_t>(errorCode);
        }

        GetStatisticsRsp::ErrorCode
        tag_invoke(const boost::json::value_to_tag<GetStatisticsRsp::ErrorCode> &,
                   const boost::json::value &jv)
        {
            switch (boost::json::value_to<uint8_t>(jv))
            {
            case 0:
                return GetStatisticsRsp::ErrorCode::noError;
            case 1:
                return GetStatisticsRsp::ErrorCode::transactionError;
            case 2:
                return GetStatisticsRsp::ErrorCode::noProvidersError;
            default:
                throw std::range_error("Bad Request");
            }
        }

    }

} // namespace pm
