#pragma once

// AFAP includes.
#include "components/worker_node/src/primitive.hpp"
#include "pm/common/src/pm_types.hpp"

namespace pm
{
    namespace primitive
    {
        // GET_STATISTICS_REQ
        struct GetStatisticsReq : public ::primitive::generic
        {
            bool reset{false};
            counter::vectorFullIDs counters;

            GetStatisticsReq(const bool reset = false,
                             const counter::vectorFullIDs &counters = {});
        };

        // GET_STATISTICS_RSP
        struct GetStatisticsRsp : public ::primitive::generic
        {

            enum class ErrorCode : uint8_t
            {
                noError = 0,
                transactionError = 1,
                noProvidersError = 2
            };

            counter::container counters;
            ErrorCode errorCode;

            explicit GetStatisticsRsp(const counter::container &counters = {},
                                      const ErrorCode &errorCode = ErrorCode::noError);
        };

        void tag_invoke(boost::json::value_from_tag,
                        boost::json::value &jv,
                        const GetStatisticsRsp::ErrorCode &c);
        GetStatisticsRsp::ErrorCode
        tag_invoke(const boost::json::value_to_tag<GetStatisticsRsp::ErrorCode> &,
                   const boost::json::value &jv);

        // PM_PROVIDER_REGISTER_REQ
        struct ProviderRegisterReq : public ::primitive::generic
        {
            counter::vectorIDs counters;

            explicit ProviderRegisterReq(const counter::vectorIDs &counters = {});
        };

        // GET_COUNTERS_REQ
        struct GetCountersReq : public ::primitive::generic
        {
            bool reset = false;
            counter::vectorFullIDs counters;

            GetCountersReq(const bool reset = false, const counter::vectorFullIDs &counters = {});
        };

        // GET_COUNTERS_RSP
        struct GetCountersRsp : public ::primitive::generic
        {
            counter::container counters;

            explicit GetCountersRsp(const counter::container &counters = {});
        };
    } // namespace primitive

} // namespace pm
