#include "pm/common/src/pm_types.hpp"

#ifndef NO_VALUE
#define NO_VALUE 0xFFFFFFFF
#endif
#define TEXT_NO_VALUE "-1"

namespace pm
{
    namespace counter
    {

        constexpr value_t zero = 0;
        constexpr value_t max_value = NO_VALUE - 1;
        constexpr value_t no_value = NO_VALUE;

        value::value() : m_value(zero)
        {
        }
        value::value(value_t val) : m_value(val)
        {
        }

        value value::no_value()
        {
            return value(pm::counter::no_value);
        }

        value &value::operator+=(const value &delta)
        {
            *this = *this + delta;
            return *this;
        }

        value value::operator+(const value &delta) const
        {

            if (delta.m_value != pm::counter::no_value)
            {
                if (m_value > (max_value - delta.m_value))
                {
                    // MAX_VALUE + 1 = 1.
                    return value(delta.m_value - (max_value - m_value));
                }
                else
                {
                    return value(m_value + delta.m_value);
                }
            }
            return value(m_value);
        }

        bool value::operator==(const value &v) const
        {
            return (m_value == v.m_value);
        }

        std::ostream &operator<<(std::ostream &os, const value &v)
        {
            if (v.m_value == no_value)
                os << TEXT_NO_VALUE;
            else
                os << v.m_value;
            return os;
        }

        void tag_invoke(boost::json::value_from_tag, boost::json::value &jv, value const &v)
        {
            jv.emplace_uint64() = v.m_value;
        }

        value tag_invoke(boost::json::value_to_tag<value>, boost::json::value const &jv)
        {
            return value(boost::json::value_to<uint32_t>(jv));
        }
    }
}