#include "pm_job.hpp"

namespace pm
{

    job::job(boost::asio::io_context &ioc,
             const std::string &name,
             job::period duration,
             const counter::vectorFullIDs &counters) : m_ioc(ioc),
                                                       m_name(name),
                                                       m_period(duration),
                                                       m_counters(counters)
    {
    }

    job::~job()
    {
        stop();
    }

    const std::string &job::name() const
    {
        return m_name;
    }

    const job::period &job::get_period() const
    {
        return m_period;
    }

    const counter::vectorFullIDs &job::counters() const
    {
        return m_counters;
    }

    void job::start()
    {
        m_timer = std::make_shared<timers::periodic>(m_ioc,
                                                     "PM_JOB_" + m_name,
                                                     m_period);
        m_timer->::timers::interfaces::manager::client1::bind(shared_from_this());
        m_timer->start();
    }

    void job::stop()
    {
        ::timers::interfaces::manager::server::stop_server();
        if (m_timer != nullptr)
        {
            m_timer->stop();
            m_timer.reset();
        }
    }

    void job::manage_timeout(const ::timers::id &id,
                             const ::timers::time_point &start_time)
    {
        interfaces::notifier::client::notify_timeout(*this, start_time);
    }

    void job::manage_stopped(const ::timers::id &id)
    {
        m_timer.reset();
    }

} // namespace pm
