#pragma once

// AFAP includes.
#include "components/config/src/config_manager.hpp"
#include "components/worker_node/src/sender.hpp"
#include "components/worker_node/src/transaction_manager.hpp"
#include "pm/common/src/pm_primitive.hpp"
#include "pm/manager/src/pm_job.hpp"

namespace pm
{
    namespace transaction
    {
        class data : public ::transaction::base_data
        {
        public:
            std::string m_jobName;
            ::timers::time_point m_startTime;

            data() = default;
            explicit data(const std::string &job_name,
                          const ::timers::time_point &startTime)
                : m_jobName(job_name),
                  m_startTime(startTime)
            {
            }
            data(data &&other) = default;
            data &operator=(const data &other) = default;
        };

        using manager = ::transaction::manager<data>;
    } // namespace transaction

    class manager : public interfaces::notifier::server,
                    public workers::interfaces::sender::client
    {
    public:
        manager(const TypeInst &nodeId,
                ::transaction::manager_id trans_mngr_id,
                boost::asio::io_context &ioc,
                const configuration::manager &config);
        ~manager() override;

        std::shared_ptr<job> create_job(const std::string &job_name);
        errors generate_jobs(const std::vector<std::string> &jobs_list);
        std::shared_ptr<job> get_job(const std::string &job_name);
        void start_all_jobs();
        void stop_and_del_all_jobs();
        bool generate_report(std::shared_ptr<primitive::GetStatisticsRsp> rsp,
                             std::string &report_name);

    private:
        void start_job(std::shared_ptr<job> a_job) const;
        void stop_job(std::shared_ptr<job> a_job) const;
        void notify_timeout(const pm::job &job,
                            const ::timers::time_point &start_time) override;

    protected:
        // Protector of shared data in multiple threads
        mutable std::recursive_mutex m_guard;
        // Context for asynchronous operations
        boost::asio::io_context &m_ioc;
        // Reference to configuration object
        const configuration::manager &m_config;
        // Own transaction manager
        transaction::manager m_trans_mngr;
        // Storage for pm jobs
        std::unordered_map<std::string, std::shared_ptr<job>> m_jobs;

        Uvr::UvrId_t m_uvrId;
    };
} // namespace pm
