// Own include.
#include "ArgsParser.hpp"

// AFAP includes.
#include "components/log/src/Log.hpp"

namespace
{
    std::string getNextArgument(int &argC, char **&argV)
    {
        --argC;
        ++argV;

        if (argC <= 0)
        {
            return "";
        }

        return argV[0];
    }
}

namespace Cli
{
    ArgsParser::ArgsParser(int argC, char **argV)
        : m_isOk(false),
          m_isToPrintHelp(false),
          m_uvrAddress(""),
          m_uvrPort("")
    {
        auto currentArg = getNextArgument(argC, argV);
        if (currentArg.empty())
        {
            LOGE(s_logTag, "Not enough arguments.");
            return;
        }

        // Checking if the first argument is a help key.
        if (currentArg == "-h" || currentArg == "-?" || currentArg == "-help" || currentArg == "--help")
        {
            m_isToPrintHelp = true;
            m_isOk = true;
            return;
        }

        // If the first argument is not a help key, then it is assumed to be the
        // address of the UVR.
        m_uvrAddress = currentArg;

        // Get the second argument. It must be a port of the UVR.
        m_uvrPort = getNextArgument(argC, argV);
        if (m_uvrPort.empty())
        {
            LOGE(s_logTag, "UVR port is missing.");
            return;
        }

        // The third argument should be a command.
        m_cmd = getNextArgument(argC, argV);
        if (m_cmd.empty())
        {
            LOGE(s_logTag, "Command is missing.");
            return;
        }

        // All subsequent arguments must be command arguments. The command may have no arguments.
        currentArg = getNextArgument(argC, argV);
        while (!currentArg.empty())
        {
            m_cmdArgs.emplace_back(std::move(currentArg));
            currentArg = getNextArgument(argC, argV);
        }

        m_isOk = true;
    }

    bool ArgsParser::isOk() const
    {
        return m_isOk;
    }

    bool ArgsParser::isToPrintHelp() const
    {
        return m_isToPrintHelp;
    }

    const std::string & ArgsParser::getUvrAddress() const
    {
        return m_uvrAddress;
    }

    const std::string & ArgsParser::getUvrPort() const
    {
        return m_uvrPort;
    }

    const std::string & ArgsParser::getCmd() const
    {
        return m_cmd;
    }

    const Strings & ArgsParser::getCmdArgs() const
    {
        return m_cmdArgs;
    }

} // namespace Cli
