#pragma once

// AFAP includes.
#include "components/db/src/client.hpp"
#include "pm/provider/src/pm_node.hpp"
#include "vfc/src/matcher.hpp"
#include "vfc/src/transaction.hpp"

namespace vfc
{
    class node : public pm::node,
                 public db::interfaces::tables::client,
                 public db::interfaces::tables::incidents::client,
                 public interfaces::verifier::server
    {
    protected:
        void start_node_timers();

        void prepareCounters() override;

        void handleSpecificPrimitive(const primitive::GenericPtr &prim) override;
        void handleVerifyCallQuery(const primitive::GenericPtr &prim);

        void handle_timeout(const ::timers::id &id,
                            const ::timers::time_point &start_time) override;

        // vfc::interfaces::verifier::server interface implementation
        void doTimeoutVerdict(transaction::vf_req::data &data,
                              ::transaction::id transaction_id) override;

        void doVerdict(const transaction::vf_req::data &data,
                       const ::transaction::id transactionId,
                       const Uvr::VerificationVerdict_t verificationVerdict,
                       const Uvr::VerificationVerdict_t incidentVerdict = Uvr::VerificationVerdict::undefined,
                       const Uvr::UvrId_t idUvrT = Uvr::UvrId::undefined);

        matcher m_matcher;
        transaction::vf_req::manager m_vf_req_mngr;
        std::shared_ptr<::db::client> m_db_client_p;
        timers::id m_transactions_watchdog_timerId;

    public:
        explicit node(const TypeInst &ownInst,
                      boost::asio::io_context &ioc,
                      const configuration::manager &config);
        ~node() override;

        void start(void) override;
    };

    using api = workers::api<node, workers::type::VFC, workers::max_num_of_insts>;

} // namespace vfc
