#include "vfc/src/transaction.hpp"

namespace vfc
{

    namespace transaction
    {

        namespace vf_req
        {

            data::data(const Uvr::MetaInfo::BitFlags_t metaInfoFlags,
                       const uint16_t &idUvrT,
                       ::primitive::VerifyCallQueryPtr query)
                : m_metaInfoFlags(metaInfoFlags),
                  m_idUvrT(idUvrT),
                  m_query(query)
            {
            }
            manager::manager(const TypeInst &ownInst,
                             ::transaction::manager_id mngrId,
                             Uvr::UvrId_t uvrId)
                : ::transaction::manager<data>(ownInst, mngrId, uvrId)
            {
            }

            ::transaction::id manager::openTransaction(data &&value,
                                                       ::transaction::errors &result,
                                                       ::transaction::id key)
            {
                std::lock_guard<std::recursive_mutex> guard(m_guard);
                m_watchdogs.emplace_back();
                watchdog *dog = &m_watchdogs.back();
                value.m_watchdog = dog;

                ::transaction::id ret =
                    ::transaction::manager<data>::openTransaction(std::move(value), result, key);
                if (result == ::transaction::errors::FAIL)
                {
                    m_watchdogs.pop_back();
                }
                else
                {
                    dog->m_transaction_id = ret;
                    dog->m_is_active = true;
                }
                return ret;
            }

            data manager::closeTransaction(::transaction::id key,
                                           ::transaction::errors &result)
            {
                std::lock_guard<std::recursive_mutex> guard(m_guard);
                data ret = ::transaction::manager<data>::closeTransaction(key, result);
                if (result == ::transaction::errors::OK)
                {
                    ret.m_watchdog->m_is_active = false;
                }
                return ret;
            }

            void manager::check_watchdogs()
            {
                std::lock_guard<std::recursive_mutex> guard(m_guard);
                timers::time_point deadline = timers::now() - timers::time_point(0, MS(490));
                while (!m_watchdogs.empty())
                {
                    auto dog = m_watchdogs.begin();
                    if (dog->m_is_active)
                    {
                        if (deadline > dog->m_start)
                        {
                            ::transaction::errors result;
                            data incident = closeTransaction(dog->m_transaction_id, result);
                            if (result == ::transaction::errors::OK)
                            {
                                interfaces::verifier::client::doTimeoutVerdict(incident,
                                                                               dog->m_transaction_id);
                            }
                            else
                            {
                                LOGWARN(" Failed to close timed-out transaction with",
                                        " transaction_id=", dog->m_transaction_id);
                            }
                        }
                        else
                        {
                            LOGT(" No more timed-out transactions");
                            break;
                        }
                    }
                    else
                    {
                        m_watchdogs.erase(dog);
                    }
                }
            }

            watchdog::watchdog()
            {
                timers::now(m_start);
            }

        } // namespace vf_req

    } // namespace transaction

} // namespace vfc
