#pragma once

#define HTTP_REQUEST_TAG "request"
#define HTTP_RESPONSE_TAG "response"

#define HTTP_SERVICEKEY_TAG "serviceKey"
#define HTTP_UVRO_TAG "uvrO"
#define HTTP_UVRT_TAG "uvrT"
#define HTTP_SESSIONID_TAG "sessionId"
#define HTTP_DATA_TAG "data"
#define HTTP_OPCODE_TAG "opcode"
#define HTTP_ERROR_TAG "error"
#define HTTP_ERRORCODE_TAG "errorCode"
#define HTTP_ERRORSIGNATURE_TAG "errorSignature"

#define HTTP_SERVICEKEY_PLAIN 80
#define HTTP_SERVICEKEY_CYPHR 81

#define HTTP_OPCODE_REQUEST "Request"
#define HTTP_OPCODE_CONTINUE "Continue"
#define HTTP_OPCODE_RELEASE "Release"
#define HTTP_OPCODE_ERROR "Error"

#define HTTP_VERIFY_CALL_TARGET "/VerifyCallState"

///< Protocol UVR-HUB error codes
#define HTTP_UVR_ERRORCODE_ILLEGAL_VALUE 1
#define HTTP_UVR_ERRORCODE_ILLEGAL_PARAM 2
#define HTTP_UVR_ERRORCODE_MISSING_PARAM 3
#define HTTP_UVR_ERRORCODE_OVERLOAD 4
#define HTTP_UVR_NUMBER_OUT_OF_SERVICE 5
#define HTTP_UVR_OUT_OF_SERVICE 6
#define HTTP_UVR_ERRORCODE_SYSTEM_ERROR 14
#define HTTP_HUB_ERRORCODE_UVR_IS_NOT_FOUND 16
#define HTTP_HUB_ERRORCODE_UVR_ROUTING_ERROR 17
#define HTTP_HUB_ERRORCODE_TIMEOUT 18
#define HTTP_HUB_ERRORCODE_ORIGIN_ILLEGAL_VALUE 20
#define HTTP_HUB_ERRORCODE_ORIGIN_ILLEGAL_PARAM 21
#define HTTP_HUB_ERRORCODE_ORIGIN_MISSING_PARAM 22
#define HTTP_HUB_ERRORCODE_REMOTE_ILLEGAL_VALUE 23
#define HTTP_HUB_ERRORCODE_REMOTE_ILLEGAL_PARAM 24
#define HTTP_HUB_ERRORCODE_REMOTE_MISSING_PARAM 25
#define HTTP_HUB_ERRORCODE_OVERLOAD 30
#define HTTP_HUB_ERRORCODE_SYSTEM_ERROR 40

#define HTTP_REPLY_BAD_REQUEST(_connId, _target, _errorText)                    \
    {                                                                           \
        LOGW(m_ownTypeInst,                                                     \
             " Bad HTTP request (connId=", _connId, ";target=", _target, "): ", \
             _errorText);                                                       \
        m_hubMngr.sendReply(http::messages::result::bad_request,                \
                            _connId,                                            \
                            _errorText);                                        \
        increaseCounter(pm::counter::http::bad_request,                         \
                        ::pm::counter::value(1));                               \
        increaseCounter(pm::counter::http::gen_bad_request_reply,               \
                        ::pm::counter::value(1));                               \
    }

#define HTTP_REPLY_NOT_FOUND(_connId, _target, _errorText)                      \
    {                                                                           \
        LOGW(m_ownTypeInst,                                                     \
             " Bad HTTP request (connId=", _connId, ";target=", _target, "): ", \
             _errorText);                                                       \
        m_hubMngr.sendReply(http::messages::result::not_found,                  \
                            _connId,                                            \
                            _errorText);                                        \
        increaseCounter(pm::counter::http::not_found,                           \
                        ::pm::counter::value(1));                               \
    }

#define HTTP_RESPONSE_ERROR(_connId, _target, _errorCode, _reqValPtr, _errorText)    \
    {                                                                                \
        boost::json::object errorBody;                                               \
        boost::json::object &rspObj = errorBody[HTTP_RESPONSE_TAG].emplace_object(); \
        boost::json::error_code err;                                                 \
        const boost::json::value *fValPtr;                                           \
        fValPtr = _reqValPtr->find_pointer("/" HTTP_SERVICEKEY_TAG, err);            \
        if (!err)                                                                    \
            rspObj[HTTP_SERVICEKEY_TAG] = *fValPtr;                                  \
        fValPtr = _reqValPtr->find_pointer("/" HTTP_UVRO_TAG, err);                  \
        if (!err)                                                                    \
            rspObj[HTTP_UVRO_TAG] = *fValPtr;                                        \
        fValPtr = _reqValPtr->find_pointer("/" HTTP_UVRT_TAG, err);                  \
        if (!err)                                                                    \
            rspObj[HTTP_UVRT_TAG] = *fValPtr;                                        \
        fValPtr = _reqValPtr->find_pointer("/" HTTP_SESSIONID_TAG, err);             \
        if (!err)                                                                    \
            rspObj[HTTP_SESSIONID_TAG] = *fValPtr;                                   \
        rspObj[HTTP_OPCODE_TAG] = HTTP_OPCODE_ERROR;                                 \
        errorBody[HTTP_ERROR_TAG].emplace_object()[HTTP_ERRORCODE_TAG] = _errorCode; \
        LOGW(m_ownTypeInst,                                                          \
             " Bad HTTP request (connId=", _connId,                                  \
             ";target=", _target, "): ",                                             \
             _errorText);                                                            \
        m_hubMngr.sendReply(http::messages::result::ok,                              \
                            _connId,                                                 \
                            boost::json::serialize(errorBody));                      \
        increaseCounter(pm::counter::http::bad_request,                              \
                        ::pm::counter::value(1));                                    \
        increaseCounter(pm::counter::http::gen_error_response,                       \
                        ::pm::counter::value(1));                                    \
    }

#define GET_MANDATORY_HTTP_REQUEST_BODY_FIELD(_connId, _target, _reqValPtr, _bodyVal, _path, _type, _field)                 \
    _type _field;                                                                                                           \
    {                                                                                                                       \
        boost::json::value val;                                                                                             \
        try                                                                                                                 \
        {                                                                                                                   \
            val = _bodyVal.at_pointer(_path);                                                                               \
            _field = boost::json::value_to<_type>(val);                                                                     \
        }                                                                                                                   \
        catch (const boost::json::system_error &ex)                                                                         \
        {                                                                                                                   \
            std::string errText = std::string("mandatory field ") + _path + (val.is_null() ? " is absent" : " is invalid"); \
            HTTP_RESPONSE_ERROR(_connId, _target,                                                                           \
                                val.is_null()                                                                               \
                                    ? HTTP_UVR_ERRORCODE_MISSING_PARAM                                                      \
                                    : HTTP_UVR_ERRORCODE_ILLEGAL_VALUE,                                                     \
                                _reqValPtr, errText);                                                                       \
            return;                                                                                                         \
        }                                                                                                                   \
    }

#define GET_MANDATORY_HTTP_RESPONSE_BODY_FIELD(_jval, _path, _type, _field) \
    _type _field;                                                           \
    {                                                                       \
        boost::json::value val;                                             \
        try                                                                 \
        {                                                                   \
            val = _jval.at_pointer(_path);                                  \
            _field = boost::json::value_to<_type>(val);                     \
        }                                                                   \
        catch (const boost::json::system_error &ex)                         \
        {                                                                   \
            LOGW(m_ownTypeInst,                                             \
                 " Bad HTTP response (connId=", connId,                     \
                 ";target=", target, ";res=", result,                       \
                 "): mandatory field \"", _path,                            \
                 (val.is_null() ? "\" is absent" : "\" is invalid"));       \
            increaseCounter(pm::counter::http::bad_response,                \
                            ::pm::counter::value(1));                       \
            /* ERR1 to be increased after timeout from VFC */               \
            return;                                                         \
        }                                                                   \
    }

#define GET_OPTIONAL_HTTP_MESSAGE_BODY_FIELD(_tag, _type, _field) \
    _type _field;                                                 \
    {                                                             \
        boost::json::value *val_p = jbody.if_contains(_tag);      \
        if (val_p != nullptr)                                     \
        {                                                         \
            _field = boost::json::value_to<_type>(*val_p);        \
        }                                                         \
    }

#define CHECK_HTTP_ERROR_SIGNATURE(_bodyVal, _errCode, _addr)                                            \
    {                                                                                                    \
        GET_MANDATORY_HTTP_RESPONSE_BODY_FIELD(_bodyVal, "/" HTTP_ERROR_TAG "/" HTTP_ERRORSIGNATURE_TAG, \
                                               std::string, errSignature);                               \
        Uvr::Gt_t hubId;                                                                                 \
        if (!m_hubMngr.getHubIdByIpAddress(_addr, hubId))                                                \
        {                                                                                                \
            LOGW(m_ownTypeInst,                                                                          \
                 " Cannot check signature: http response with error code=", _errCode,                    \
                 " received from unknown HUB with IP address=", _addr);                                  \
            increaseCounter(pm::counter::http::bad_response,                                             \
                            ::pm::counter::value(1));                                                    \
            /* ERR1 to be increased after timeout from VFC */                                            \
            return;                                                                                      \
        }                                                                                                \
        if (!checkHubSignature(hubId, errSignature))                                                     \
        {                                                                                                \
            increaseCounter(pm::counter::http::bad_response,                                             \
                            ::pm::counter::value(1));                                                    \
            /* ERR1 to be increased after timeout from VFC */                                            \
            return;                                                                                      \
        }                                                                                                \
    }
