#include "generic_param.hpp"
#include "utils.hpp"

// STL includes.
#include <cstring>
#include <string.h>

namespace sip
{

    GenericParam::GenericParam(std::string &&pname, std::string &&pvalue)
    {
        m_name = std::move(pname);
        utils::doClearLws(m_name);
        m_value = std::move(pvalue);

        if (m_value != "")
        {
            utils::doClearLws(m_value);
        }
    }

    GenericParam *GenericParamList::match(const std::string &name,
                                          int &errorCode)
    {
        if (name.empty())
        {
            errorCode = BADPARAMETER;
            return nullptr;
        }

        for (auto it = begin(); it != end(); it++)
        {
            if ((name.length() == it->m_name.length()) &&
                strncasecmp(it->m_name.c_str(), name.c_str(), name.length()))
            {
                errorCode = SUCCESS;
                return &(*it);
            }
        }

        errorCode = UNDEFINED_ERROR;
        return nullptr;
    }

    int GenericParamList::doParse(const char *params)
    {
        std::string pname;
        std::string pvalue;

        const char *comma;
        const char *equal;
        const char *startquote;
        const char *endquote;

        /* find '=' wich is the separator for one param */
        /* find ';' wich is the separator for multiple params */

        equal = utils::findSeparatorAfter(params + 1, '=', ';');
        comma = std::strchr(params + 1, ';');

        /* If comma points after value start quote, move it to after end quote */
        if (equal != nullptr)
        {
            const char *tmp;

            startquote = nullptr;

            for (tmp = equal + 1; tmp[0] == ' '; tmp++)
            {
                // Just iterate it over
            }

            if (tmp[0] == '"')
                startquote = tmp;

            if (startquote != nullptr && comma > startquote)
            {
                comma = nullptr;
                endquote = utils::doFindQuote(startquote + 1);

                if (endquote)
                    comma = std::strchr(endquote, ';');
            }
        }

        while (comma != nullptr)
        {
            if (equal == nullptr)
            {
                equal = comma;
                pvalue = "";
            }
            else
            {
                const char *tmp;

                /* check for nullptr param with an '=' character */
                tmp = equal + 1;
                tmp += strspn(tmp, "\t ");
                pvalue = "";

                if (*tmp != ',' && *tmp != '\0')
                {
                    if (comma - equal < 2)
                    {
                        return SYNTAXERROR;
                    }

                    pvalue = std::string(equal + 1, comma - equal - 1);
                }
            }

            if (equal - params < 2)
            {
                return SYNTAXERROR;
            }

            pname = std::string(params + 1, equal - params - 1);

            emplace_back(std::move(pname), std::move(pvalue));

            params = comma;
            equal = utils::findSeparatorAfter(params + 1, '=', ';');
            comma = std::strchr(params + 1, ';');

            /* If comma points after value start quote, move it to after end quote */
            if (equal != nullptr)
            {
                const char *tmp;

                startquote = nullptr;

                for (tmp = equal + 1; tmp[0] == ' '; tmp++)
                {
                    // Just iterate it over
                }

                if (tmp[0] == '"')
                {
                    startquote = tmp;
                }

                if (startquote != nullptr && comma > startquote)
                {
                    comma = nullptr;
                    endquote = utils::doFindQuote(startquote + 1);

                    if (endquote)
                        comma = std::strchr(endquote, ';');
                }
            }
        }

        comma = params + std::strlen(params);

        if (equal == nullptr)
        {
            equal = comma;
            pvalue = "";

            if (equal - params < 2)
            {
                return SUCCESS;
            }
        }
        else
        {
            const char *tmp;

            tmp = equal + 1;
            tmp += strspn(tmp, "\t ");
            pvalue = "";

            if (*tmp != ',' && *tmp != '\0')
            {
                if (comma - equal < 2)
                {
                    return SYNTAXERROR;
                }

                pvalue = std::string(equal + 1, comma - equal - 1);
            }
        }

        if (equal - params < 2)
        {
            return SYNTAXERROR;
        }

        pname = std::string(params + 1, equal - params - 1);

        emplace_back(std::move(pname), std::move(pvalue));

        return SUCCESS;
    }

} // namespace sip
