#include "header_content_type.hpp"
#include "utils.hpp"

// STL includes.
#include <cstring>

namespace sip::header
{

    ContentType::ContentType()
        : Abstract(std::string("Content-Type"))
    {
    }

    std::string ContentType::valueToStr(int &errorCode)
    {
        if ((m_type.empty()) || (m_subtype.empty()))
        {
            errorCode = BADPARAMETER;
            return "";
        }

        std::string ret = m_type + "/" + m_subtype;

        for (auto it = m_genParams.begin(); it != m_genParams.end(); it++)
        {
            ret += "; " + it->m_name + "=" + it->m_value;
        }

        errorCode = SUCCESS;
        return ret;
    }

    int ContentType::doParse(const char *hvalue)
    {
        const char *subtype;
        const char *contentTypeParams;
        int errorCode;

        if (hvalue == nullptr || hvalue[0] == '\0')
        {
            return SUCCESS;
        }

        subtype = std::strchr(hvalue, '/');
        contentTypeParams = std::strchr(hvalue, ';');

        if (subtype == nullptr)
        {
            return SYNTAXERROR;
        }

        if (contentTypeParams != nullptr)
        {
            errorCode = m_genParams.doParse(contentTypeParams);

            if (errorCode != SUCCESS)
            {
                return errorCode;
            }
        }
        else
        {
            contentTypeParams = subtype + std::strlen(subtype);
        }

        if (subtype - hvalue + 1 < 2)
        {
            return SYNTAXERROR;
        }

        m_type = utils::doCopyAndClearLws(hvalue, subtype - hvalue);

        if (contentTypeParams - subtype < 2)
        {
            return SYNTAXERROR;
        }

        m_subtype = utils::doCopyAndClearLws(subtype + 1,
                                             contentTypeParams - subtype - 1);

        return SUCCESS;
    }

} // namespace sip::header
