#include "header_cseq.hpp"

#include <cstring>

namespace sip::header
{

    CSeq::CSeq()
        : Abstract(std::string("CSeq"))
    {
    }

    int CSeq::doParse(const char *hvalue)
    {
        if (hvalue == nullptr)
        {
            return BADPARAMETER;
        }

        const char *method = NULL;
        const char *end = NULL;

        method = std::strchr(hvalue, ' ');

        if (method == NULL)
        {
            return SYNTAXERROR;
        }

        end = hvalue + std::strlen(hvalue);

        if (method - hvalue < 1)
        {
            return SYNTAXERROR;
        }

        if (end - (method + 1) < 2)
        {
            return SYNTAXERROR;
        }

        m_number = std::string(hvalue, method - hvalue);
        m_method = std::string(method + 1, end - (method + 1));

        return SUCCESS;
    }

    std::string CSeq::valueToStr(int &errorCode)
    {
        if (m_number.empty() || m_method.empty())
        {
            errorCode = BADPARAMETER;
            return std::string();
        }

        errorCode = SUCCESS;
        return m_number + " " + m_method;
    }

    int CSeq::getValue()
    {
        return std::strtol(m_number.c_str(), nullptr, 10);
    }


} // namespace sip::header
