#include "header_from.hpp"
#include "utils.hpp"

// STL includes.
#include <cstring>

namespace sip
{

    namespace header
    {

        From::From()
            : Abstract(std::string("From"))
        {
        }

        int From::doParse(const char *hvalue)
        {
            const char *displayname = nullptr;
            const char *uri = nullptr;
            const char *uri_end = nullptr;
            const char *gen_params;
            const char *ptr;
            int i;

            if (hvalue == nullptr)
            {
                return BADPARAMETER;
            }

            ptr = hvalue;

            while (ptr[0] != '\0')
            {
                if (ptr[0] == ' ')
                {
                    ptr++;
                    continue;
                }

                if (ptr[0] == '"')
                {
                    displayname = ptr;
                }

                break;
            }

            if (ptr[0] == '\0')
            {
                return SUCCESS;
            }

            if (displayname != nullptr)
            {
                const char *second = nullptr;

                second = utils::doFindQuote(displayname + 1);

                if (second == nullptr)
                {
                    return SYNTAXERROR;
                }

                if (second - displayname + 2 >= 2)
                {
                    m_displayName = utils::doCopyAndClearLws(displayname,
                                                             second - displayname + 1);
                }

                ptr = second + 1;

                while (ptr[0] != '\0')
                {
                    if (ptr[0] == ' ')
                    {
                        ptr++;
                        continue;
                    }

                    if (ptr[0] == '<')
                    {
                        uri = ptr;
                    }

                    break;
                }

                if (uri == nullptr)
                {
                    return SYNTAXERROR;
                }

                if (ptr[1] == '\0')
                {
                    return SYNTAXERROR;
                }
            }
            else
            {
                const char *beg = ptr;

                while (ptr[0] != '\0')
                {
                    if (ptr[0] == ' ')
                    {
                        ptr++;
                        continue;
                    }

                    if (ptr[0] >= 'a' && ptr[0] <= 'z')
                    {
                        ptr++;
                        continue;
                    }

                    if (ptr[0] >= 'A' && ptr[0] <= 'Z')
                    {
                        ptr++;
                        continue;
                    }

                    if (ptr[0] >= '0' && ptr[0] <= '9')
                    {
                        ptr++;
                        continue;
                    }

                    if (ptr[0] == '-' || ptr[0] == '.' || ptr[0] == '!' ||
                        ptr[0] == '%' || ptr[0] == '*' || ptr[0] == '_' ||
                        ptr[0] == '+' || ptr[0] == '`' || ptr[0] == '\'' ||
                        ptr[0] == '~')
                    {
                        ptr++;
                        continue;
                    }

                    uri = ptr;
                    break;
                }

                if (ptr[0] == '\0' || uri == nullptr)
                {
                    return SYNTAXERROR;
                }

                if (ptr[0] == '<')
                {
                    if (ptr[1] == '\0')
                    {
                        return SYNTAXERROR;
                    }

                    if (uri - beg > 0)
                    {
                        m_displayName = utils::doCopyAndClearLws(hvalue, uri - beg);
                    }
                }
                else if (ptr[0] == ':')
                {
                    uri = beg;
                }
                else
                {
                    uri = beg;
                }
            }

            if (uri[0] == '<')
            {
                uri++;
                ptr = uri;

                uri_end = std::strchr(ptr, '>');

                if (uri_end == nullptr)
                {
                    return SYNTAXERROR;
                }

                uri_end--;
                gen_params = std::strchr(uri_end, ';');
            }

            if (uri_end == nullptr)
            {
                gen_params = std::strchr(uri, ';');

                if (gen_params != nullptr)
                {
                    uri_end = gen_params - 1;
                }
                else
                {
                    uri_end = uri + std::strlen(uri);
                }
            }

            if (gen_params != nullptr)
            {
                i = m_genParams.doParse(gen_params);

                if (i != 0)
                {
                    return i;
                }
            }

            {
                if (uri_end - uri + 2 < 7)
                {
                    return SYNTAXERROR;
                }

                std::string tmp(uri, uri_end - uri + 1);
                m_uri = std::make_shared<Uri>();
                i = m_uri->doParse(tmp.c_str());

                if (i != 0)
                {
                    m_uri.reset();
                    return i;
                }
            }
            return SUCCESS;
        }

        std::string From::valueToStr(int &errorCode)
        {
            if (m_uri.get() == nullptr)
            {
                errorCode = BADPARAMETER;
                return "";
            }

            std::string ret = m_uri->toStr(errorCode);

            if (errorCode != SUCCESS)
            {
                return "";
            }

            if (!m_displayName.empty())
            {
                ret = m_displayName + " <" + ret + ">";
            }
            else
            {
                ret = "<" + ret + ">";
            }

            for (auto it = m_genParams.begin(); it != m_genParams.end(); it++)
            {
                if (it->m_value.empty())
                {
                    ret += ";" + it->m_name;
                }
                else
                {
                    ret += ";" + it->m_name + "=" + it->m_value;
                }
            }

            errorCode = SUCCESS;
            return ret;
        }

    } // namespace header

} // namespace sip
