#include "headers.hpp"
#include "utils.hpp"

namespace sip
{

    namespace header
    {
        static const Container knownHeaders{
            {"c", {false, false, Id::contentType}},
            {"content-type", {false, false, Id::contentType}},
            {"f", {false, false, Id::from}},
            {"from", {false, false, Id::from}},
            {"l", {false, false, Id::contentLength}},
            {"content-length", {false, false, Id::contentLength}},
            {"t", {false, false, Id::to}},
            {"to", {false, false, Id::to}},
            {"v", {false, false, Id::via}},
            {"via", {false, true, Id::via}},
            {"allow", {true, false, Id::allow}},
            {"supported", {true, true, Id::supported}},
            {"allow-events", {true, true, Id::allowEvents}},
            {"cseq", {true, false, Id::cSeq}},
            {"i", {true, false, Id::callId}},
            {"call-id", {true, false, Id::callId}},
            {"diversion", {true, false, Id::diversion}},
            {"m", {true, false, Id::contact}},
            {"contact", {true, false, Id::contact}}};

        bool isCommaSeparated(std::string &name)
        {
            auto iter = knownHeaders.find(name);
            if (iter != knownHeaders.end())
            {
                return iter->second.m_isCommaSeparated;
            }

            return false;
        }

        Id getHeaderId(std::string name)
        {
            auto iter = knownHeaders.find(name);
            if (iter != knownHeaders.end())
            {
                return iter->second.m_id;
            }

            return Id::generic;
        }

        Generic::Generic(std::string &&name,
                         std::string &&value)
            : Abstract(std::move(name)),
              m_value(std::move(value))
        {
        }
        int Generic::doParse(const char *hvalue)
        {
            return SUCCESS;
        }
        std::string Generic::valueToStr(int &errorCode)
        {
            errorCode = SUCCESS;
            return m_value;
        }

        std::string Abstract::toStr(int &errorCode)
        {
            std::string ret;
            errorCode = SUCCESS;

            if (m_name.empty())
            {
                errorCode = BADPARAMETER;
                return "";
            }

            ret += m_name + ": ";

            std::string tmp;
            ret += valueToStr(errorCode);
            if (errorCode != SUCCESS)
            {
                return "";
            }

            ret += CRLF;

            return ret;
        }

        std::string SingleLineHeader::toStr(int &errorCode)
        {
            if (get() != nullptr)
            {
                return get()->toStr(errorCode);
            }
            else
            {
                errorCode = SUCCESS;
                return "";
            }
        }

        std::string MultipleLineHeader::toStr(int &errorCode)
        {
            std::string ret;
            errorCode = SUCCESS;

            for (auto it = begin(); it != end(); it++)
            {
                ret += (*it)->toStr(errorCode);

                if (errorCode != SUCCESS)
                {
                    return "";
                }
            }

            return ret;
        }

    } // namespace header

} // namespace sip
