#pragma once

#include "generic_param.hpp"

// STL includes.
#include <memory>
#include <string>
#include <unordered_map>

#define CR "\r"
#define LF "\n"
#define CRLF CR LF

#define RETURN_STR_FAIL(E) \
    errorCode = E;         \
    return "";

#define RETURN_STR(E, R) \
    errorCode = E;       \
    return R;

namespace sip
{

    namespace header
    {

        enum class Id
        {
            contentType,
            contentLength,
            from,
            to,
            via,
            allow,
            supported,
            allowEvents,
            cSeq,
            callId,
            diversion,
            contact,
            generic
        };

        struct Properties
        {
            bool m_ignoredWhenInvalid;
            bool m_isCommaSeparated;
            Id m_id;
        };

        using Container = std::unordered_map<std::string, Properties>;

        class Abstract
        {
        public:
            explicit Abstract(std::string &&name)
                : m_name(std::move(name))
            {
            }
            virtual ~Abstract() = default;
            virtual int doParse(const char *hvalue) = 0;
            virtual std::string toStr(int &errorCode) final;
            virtual std::string valueToStr(int &errorCode) = 0;

            std::string m_name;
        };
        using AbstractPtr = std::shared_ptr<Abstract>;
        using AbstractPtrList = std::list<AbstractPtr>;

        class Generic : public Abstract
        {
        public:
            Generic(std::string &&name,
                    std::string &&value);

            int doParse(const char *hvalue) override;
            std::string valueToStr(int &errorCode) override;

            std::string m_value;
        };
        using GenericPtr = std::shared_ptr<Generic>;
        using GenericPtrList = std::list<GenericPtr>;

        class AbstractContainer
        {
        public:
            virtual ~AbstractContainer() = default;
            virtual std::string toStr(int &errorCode) = 0;
        };
        class SingleLineHeader : public AbstractContainer,
                                 public AbstractPtr
        {
        public:
            SingleLineHeader &operator=(AbstractPtr hdr)
            {
                AbstractPtr::operator=(hdr);
                return (*this);
            }

        protected:
            std::string toStr(int &errorCode) override;
        };
        class MultipleLineHeader : public AbstractContainer,
                                   public AbstractPtrList
        {
        public:
            std::string toStr(int &errorCode) override;
        };

        bool isCommaSeparated(std::string &name);
        Id getHeaderId(std::string name);

    }

} // namespace sip
