#pragma once

#include "trunk_udp.hpp"

// AFAP includes.
#include "components/support/interfaces_template.hpp"
#include "timers/src/timer_periodic.hpp"
#include "timers/src/timer_manager.hpp"

// STL includes.
#include <unordered_map>

namespace sip
{
    namespace udp
    {
        namespace transport
        {

            using CallbackUdp_t = trunk::CallbackUdp_t;

            namespace interfaces
            {

                INTERFACE_S(handler,
                            INTERFACE_SERVER(void, handleMessageUdp, const trunk::Socket &, const MessagePtr &);)
                INTERFACE_C1(handler,
                             INTERFACE_CLIENT1(void, handleMessageUdp, const trunk::Socket &, const MessagePtr &);)
            } // namespace interfaces

            class Manager : public std::enable_shared_from_this<Manager>,
                            public interfaces::handler::client1,
                            public trunk::interfaces::controller::server
            {
                using Mutex = std::recursive_mutex;
                using Guard = std::lock_guard<Mutex>;

            public:
                explicit Manager(boost::asio::io_context &ioc,
                                 const Log::Tag &logTag = Log::Tag(""));
                ~Manager() override;

                void setSocketRcvBufSize(const int val);

                void start();
                virtual void stop();

                trunk::Id addSocket(const network::address &address,
                                    const std::string &port,
                                    const ::sip::trunk::Role &role);
                void send(const trunk::Id &id, Message &msg) const
                {
                    LOGE(m_logTag, "Remote endpoint should be specified to send UDP data");
                }
                void send(const trunk::Id &id, Message &msg,
                          const std::string &remIp,
                          const std::string &port) const;
                void send(const trunk::Id &id, Message &msg,
                          const std::string &remIp,
                          const std::string &remPort,
                          CallbackUdp_t callback) const;
                void send(const trunk::Id &id,
                          const std::string &remIp,
                          const std::string &port,
                          const std::string &msg) const;
                void send(const trunk::Id &id,
                          const std::string &remIp,
                          const std::string &remPort,
                          const std::string &msg,
                          CallbackUdp_t callback) const;

                std::string getLocalAddress(const ::sip::trunk::Reference &ref) const;
                std::string getLocalPort(const ::sip::trunk::Reference &ref) const;

            protected:
                boost::asio::io_context &m_ioc; ///< Context for asynchronous operations
                const Log::Tag m_logTag;

            private:
                void controllMessage(trunk::Socket *connPtr, const MessagePtr &msg) override;

                mutable Mutex m_guard;                   ///< Protector of shared data in multiple threads
                std::vector<trunk::SocketPtr> m_sockets; ///< UDP socket for sending and receiving.
                int m_socketRcvBufSize = 0;              ///< Receive buffer size (in bytes) for each socket.
            };

        } // namespace transport

    } // namespace udp

} // namespace sip
