#pragma once

#include "transport_udp.hpp"
#include "transport_tcp.hpp"

namespace sip
{
    namespace transport
    {

        namespace interfaces
        {

            INTERFACE_S(handler,
                        INTERFACE_SERVER(void, handleMessage, const trunk::Reference &, const MessagePtr &);
                        INTERFACE_SERVER(void, handleTrunkTcpUp, const trunk::Reference &);)
            INTERFACE_C1(handler,
                         INTERFACE_CLIENT1(void, handleMessage, const trunk::Reference &, const MessagePtr &);
                         INTERFACE_CLIENT1(void, handleTrunkTcpUp, const trunk::Reference &);)
        } // namespace interfaces

        class Manager : public std::enable_shared_from_this<Manager>,
                        public interfaces::handler::client1,
                        public udp::transport::interfaces::handler::server,
                        public tcp::transport::interfaces::handler::server
        {
        public:
            explicit Manager(boost::asio::io_context &ioc,
                             const Log::Tag &logTag = Log::Tag(""));

            virtual ~Manager();

            void setUdpSocketRcvBufSize(const int val);

            void start();
            void stop();

            template <typename... ColumnsTypes>
            void send(const trunk::Reference &tRef,
                      ColumnsTypes &...columnsValues) const
            {
                switch (tRef.m_ulp)
                {
                    case trunk::Protocol::UDP:
                        m_udpManager->send(tRef.m_key,
                                           columnsValues...);
                        break;
                    case trunk::Protocol::TCP:
                        m_tcpManager->send(tRef.m_key,
                                           columnsValues...);
                        break;
                }
            }

            void addListener(trunk::Protocol protocol,
                             const network::address &address,
                             const std::string &port,
                             const sip::trunk::Role &role);
            trunk::Reference addTrunkTcp(const std::string &address,
                                         const std::string &port);

            std::string getLocalAddress(const ::sip::trunk::Reference &ref) const;
            std::string getLocalPort(const ::sip::trunk::Reference &ref) const;

        protected:
            void handleMessageUdp(const udp::trunk::Socket &trunk,
                                  const MessagePtr &msg) override;
            void handleMessageTcp(const tcp::trunk::connection &trunk,
                                  const MessagePtr &msg) override;
            void handleSipConnUp(const tcp::trunk::Id &id) override;

            boost::asio::io_context &m_ioc; ///< Context for asynchronous operations
            const Log::Tag m_logTag;

            std::shared_ptr<::sip::udp::transport::Manager> m_udpManager;
            std::shared_ptr<::sip::tcp::transport::Manager> m_tcpManager;
        };

    } // namespace transport

} // namespace sip
