#pragma once

// SIP component includes.
#include "trunks.hpp"

// AFAP includes.
#include "components/support/interfaces_template.hpp"
#include "components/tools/src/Tools.hpp"

#include <boost/asio.hpp>

namespace sip::udp::trunk
{
    class Socket;
    using CallbackUdp_t = std::function<void(const boost::system::error_code, std::size_t)>;
    using SocketPtr = std::shared_ptr<Socket>;
    using Id = ::sip::trunk::Id;

    namespace interfaces
    {

        INTERFACE_S(controller,
                    INTERFACE_SERVER(void, controllMessage, Socket *, const MessagePtr &);)

        INTERFACE_C1(controller,
                     INTERFACE_CLIENT1(void, controllMessage, Socket *, const MessagePtr &);)

    } // namespace interfaces

    class Socket : public std::enable_shared_from_this<Socket>,
                   public interfaces::controller::client1,
                   public ::sip::trunk::Generic
    {
    public:
        Socket(boost::asio::io_service &ioc,
               const network::address &ipAddr,
               const std::string &port,
               const ::sip::trunk::Role &role,
               const Log::Tag &logTag = Log::Tag(""));

        ~Socket();

    public:
        void setRcvBufSize(const int val);

        void asyncSend(const std::string &remIp,
                       const std::string &port,
                       const std::string &msg,
                       CallbackUdp_t callback);

        void asyncRead();

        void cancel();

        const std::string &getLocalAddress() const;
        const std::string &getLocalPort() const;

    private:
        void readDatagram(const boost::system::error_code &err,
                          std::size_t udpMsgSize);

    private:
        using Buffer = std::unique_ptr<char[]>;

    private:
        boost::asio::ip::udp::endpoint m_fromEndpoint;
        boost::asio::ip::udp::socket m_socket;

        Buffer m_buffer;

        std::string m_localAddress;
        std::string m_localPort;

        const Log::Tag m_logTag;
    };

} // namespace sip::udp::trunk
