#pragma once

#include "generic_param.hpp"

#include <memory>
#include <string>

namespace sip
{

    class Uri
    {
        using Param = GenericParam;
        using ParamList = std::list<Param>;
        using Header = Param;
        using HeaderList = std::list<Header>;

    public:
        int doParse(const char *buf);
        std::string toStr(int &errorCode);

    protected:
        int doParseHeaders(const char *headers);
        int doParseParams(const std::string &paramsStr);

        std::string escapeUserinfo(const std::string &string) const;
        std::string escapePassword(const std::string &string) const;
        std::string escapeParam(const std::string &string) const;
        std::string escapeHeaderParam(const std::string &string) const;

    public:
        std::string m_scheme;   ///< Uri Scheme (sip or sips)
        std::string m_username; ///< Username
        std::string m_password; ///< Password
        std::string m_host;     ///< Domain
        std::string m_port;     ///< Port number
        ParamList m_params;     ///< Uri parameters
        HeaderList m_headers;   ///< Uri headers

        std::string m_string; ///< Space for other uri schemes. (http, mailto...)
    };
    using UriPtr = std::shared_ptr<Uri>;

} // namespace sip
