#pragma once

#include "message.hpp"
#include "UvrTypes.hpp"

// STL includes.
#include <string>

constexpr char g_sp{0x20};
constexpr char g_hTab{0x09};
constexpr char g_cr{0x0D};
constexpr char g_lf{0x0A};

constexpr int SIP_MESSAGE_MAX_LENGTH = 8000;

namespace sip
{

    namespace utils
    {

        void toLower(std::string &str);

        inline bool isAlpha(char in)
        {
            return ((in >= 'a' && in <= 'z')
                    || (in >= 'A' && in <= 'Z'));
        };

        inline bool isAlphaNum(char in)
        {
            return ((in >= 'a' && in <= 'z')
                    || (in >= 'A' && in <= 'Z')
                    || (in >= '0' && in <= '9'));
        }

        const char *findSeparatorAfter(const char *ch,
                                       int separatorToFind,
                                       int afterSeparator);

        const char *doFindQuote(const char *qstring);
        const std::string_view doUnQuote(const std::string &string);

        void doUnescape(std::string &string);

        void doClearLws(std::string &word);
        std::string doCopyAndClearLws(const char *src, size_t len);

        std::string escapeNonAsciiAndReserved(const std::string &string, const char *def);

        void replaceAllLws(char *sip_message);

        int strStr(const char *str, const char *buf,
                   const char **index_of_str, const char *endOfBuf);

        int findNextCrLf(const char *startOfHeader,
                         const char **end_of_header);

        char *findStr(const char *bufToSearchWith, size_t bufLenLeft, const char *strToFind);

        int atoi(const std::string &number);

    } // namespace utils

    namespace support
    {

        const std::string ID_SRC_SEPARATOR("*");

        std::string getNumber(const sip::Contact &contact,
                              const std::string &separator = ID_SRC_SEPARATOR);

        void parseUriNumber(Uvr::Number &uriNumber,
                            Uvr::OptionalOperatorId &operId,
                            const std::string &separator = ID_SRC_SEPARATOR);

        std::string composeInvite(const sip::Contact &from,
                                  const sip::Contact &to,
                                  const std::string &separator = ID_SRC_SEPARATOR);

        std::string composeInvite(const sip::Contact &from,
                                  const sip::Contact &to,
                                  Uvr::CallId_t &callId,
                                  int &cSeq,
                                  const std::string &separator = ID_SRC_SEPARATOR);

        std::string composeInvite(const sip::Contact &from,
                                  const sip::Contact &to,
                                  const Uvr::CallId_t &callId,
                                  int &cSeq,
                                  const std::string &separator = ID_SRC_SEPARATOR);

        std::string composeAck(const sip::Contact &from,
                               const sip::Contact &to,
                               const Uvr::CallId_t &callId,
                               const int &cSeq,
                               const std::string &separator = ID_SRC_SEPARATOR);

        std::string composeOptions(const sip::Contact &from,
                                   const sip::Contact &to,
                                   const std::string &separator = ID_SRC_SEPARATOR);
        std::string composeOptions(const sip::Contact &from,
                                   const sip::Contact &to,
                                   const Uvr::CallId_t &callId,
                                   const int &cSeq,
                                   const std::string &separator = ID_SRC_SEPARATOR);

        std::string composeCancel(const sip::Contact &from,
                                  const sip::Contact &to,
                                  const std::string &separator = ID_SRC_SEPARATOR);
        std::string composeCancel(const sip::Contact &from,
                                  const sip::Contact &to,
                                  const Uvr::CallId_t &callId,
                                  const int &cSeq,
                                  const std::string &separator = ID_SRC_SEPARATOR);

    } // namespace support

} // namespace sip
