#pragma once

#include "sip/src/message.hpp"

// Boost includes.
#include <boost/json.hpp>

// STL includes.
#include <cstdint>
#include <string>

#define CONFIG_SIP "/sip"

#define CONFIG_SIP_RESPONSE CONFIG_SIP "/response"
#define CONFIG_SIP_RESPONSE_VERIFICATION CONFIG_SIP_RESPONSE "/verification"
#define CONFIG_SIP_RESPONSE_OPTIONS CONFIG_SIP_RESPONSE "/options"

namespace modules
{
    namespace sip
    {
        /// Contains paths in config file and default values for the SIP parameters.
        namespace config
        {
            /// The port number through which SIP module communicates for
            /// incoming calls.
            constexpr auto incomingCallsPort = CONFIG_SIP "/incomingCallsPort";
            const std::string incomingCallsPortDefaultValue = "8040";

            /// The port number through which SIP module communicates for
            /// outgoing calls.
            constexpr auto outgoingCallsPort = CONFIG_SIP "/outgoingCallsPort";
            const std::string outgoingCallsPortDefaultValue = "8041";

            constexpr auto udpSocketReceiveBufferSize = CONFIG_SIP "/udpSocketReceiveBufferSize";
            const int udpSocketReceiveBufferSizeDefaultValue = 1048576;

            /// The network interface through which SIP module communicates
            constexpr auto interface = CONFIG_SIP "/interface";
            const std::string interfaceDefaultValue = "eth0";

        } // namespace config

    } // namespace sip

} // namespace modules

/// Contains paths in config file and default values for the corresponding parameters.
namespace sip
{
    namespace response
    {

        Reason tag_invoke(boost::json::value_to_tag<Reason>,
                          boost::json::value const &jv);

    } // namespace response

    namespace config
    {

        namespace response
        {
            /// The response code to be sent after verification response is received.
            namespace verification
            {
                constexpr auto success = CONFIG_SIP_RESPONSE_VERIFICATION "/success";
                const ::sip::response::Reason successDefaultValue = {402, "OK"};

                constexpr auto failure = CONFIG_SIP_RESPONSE_VERIFICATION "/failure";
                const ::sip::response::Reason failureDefaultValue = {403, "Not found"};

                constexpr auto timeout = CONFIG_SIP_RESPONSE_VERIFICATION "/timeout";
                const ::sip::response::Reason timeoutDefaultValue = {404, "Request Timeout"};

                constexpr auto notRussian = CONFIG_SIP_RESPONSE_VERIFICATION "/callingNumberNotRussian";
                const ::sip::response::Reason notRussianDefaultValue = {405, "Number is not Russian"};

                constexpr auto notServiced = CONFIG_SIP_RESPONSE_VERIFICATION "/callingNumberNotServiced";
                const ::sip::response::Reason notServicedDefaultValue = {406, "Number not serviced"};

                constexpr auto notFound = CONFIG_SIP_RESPONSE_VERIFICATION "/callingNumberNotFound";
                const ::sip::response::Reason notFoundDefaultValue = {407, "Number not found"};

            } // namespace verification

            /// The response code to be sent for outgoing calls.
            constexpr auto outgoingCall = CONFIG_SIP_RESPONSE "/outgoingCall";
            const ::sip::response::Reason outgoingCallDefaultValue = {500, "Outgoing call"};

            namespace options
            {
                constexpr auto notStarted = CONFIG_SIP_RESPONSE_OPTIONS "/notStarted";
                const ::sip::response::Reason notStartedDefaultValue = {200, ""};

            } // namespace options

        } // namespace response

        constexpr auto idSrcSeparator = CONFIG_SIP "/idSrcSeparator";
        const std::string idSrcSeparatorDefaultValue = "*";

    } // namespace config

} // namespace sip
