#pragma once

#include "Config.hpp"

// AFAP includes.
#include "modules/sip/src/transaction.hpp"
#include "components/tools/src/Tools.hpp"

namespace modules
{

    namespace sip
    {

        class node : public std::enable_shared_from_this<node>,
                     public ::sip::transport::interfaces::handler::server
        {
        public:
            explicit node(const std::string &configPath = std::string(),
                          const ::Uvr::UserId_t &apiUser = ::Uvr::undefinedUserId,
                          const Log::Tag &logTag = Log::Tag(""));

            void start(int threadPoolSize);
            void addListener(::sip::trunk::Protocol protocol,
                             const std::string &address,
                             const std::string &port,
                             const ::sip::trunk::Role &role);
            void stop();
            void wait_stop();

            void handleVerdict(const Uvr::VerificationResult &result);
            void handleCallInformationRequest(const Uvr::CallInformationRequest &request) const;

        protected:
            void handleMessage(const ::sip::trunk::Reference &id, const ::sip::MessagePtr &msg) override;
            void handleTrunkTcpUp(const ::sip::trunk::Reference &id) override;

            io_service::Context m_ioc;
            Uvr::UserId_t m_userId;
            std::shared_ptr<::sip::transport::Manager> m_transportManager;

            using BaseTransactionManager = ::transaction::manager<transaction::data>;
            class TransactionManager : public BaseTransactionManager
            {
            public:
                using BaseTransactionManager::BaseTransactionManager;

                transaction::data closeTransaction(Uvr::SessionId_t sessionId,
                                                   ::transaction::errors &result) override;

                void bind(Uvr::SessionId_t sessionId,
                          ::transaction::id);

            protected:
                std::unordered_map<Uvr::SessionId_t, ::transaction::id> m_mapUvrToSipSession;
            };

            TransactionManager
                m_vfCallsTransMngr{TypeInst(workers::type::SIPMDL, 0), 0, 0};

            const Log::Tag m_logTag;

        private:
            void sendReply(const ::sip::trunk::Reference &ref,
                           const ::sip::MessagePtr &request,
                           const ::sip::response::Reason &reason,
                           const ::sip::header::GenericPtrList &extraHeaders =
                               ::sip::header::GenericPtrList());
        };

        class api
        {
        public:
            static std::shared_ptr<node> node_p;

            static void init(const std::string &configPath = std::string(),
                             const ::Uvr::UserId_t &apiUser = ::Uvr::undefinedUserId);
            static void start(const int &numberOfThreads = 5);
            static void stop();
            static void wait_stop();
            static void addListener(::sip::trunk::Protocol protocol,
                                    const std::string &address,
                                    const std::string &port,
                                    const ::sip::trunk::Role &role);
        };

    } // namespace sip

} // namespace modules
