// Own include.
#include "ClickHouseClient.hpp"

// AFAP includes.
#include "TRecQ763Number.hpp"

namespace
{
    constexpr int s_clickHouseNumOfThreads = 1;
}

Storage::ClickHouseClient::ClickHouseClient(const Uvr::UvrId_t uvrId,
                                            const configuration::manager &config)
{
    m_dbClient = std::make_shared<::db::ClickHouseAcc>(m_ioc, uvrId, config);
    db::interfaces::tables::client::bind(m_dbClient.get());
    db::interfaces::tables::calls::client::bind(m_dbClient.get());
}

Storage::ClickHouseClient::~ClickHouseClient()
{
    m_ioc.stop();
    disconnect();
    m_dbClient.reset();
    m_ioc.wait_stop();
}

bool Storage::ClickHouseClient::connect()
{
    if (!m_dbClient->connect())
    {
        return false;
    }
    LOGI("Connection with DB is established.");

    // Start context
    m_ioc.start(s_clickHouseNumOfThreads);

    return true;
}

void Storage::ClickHouseClient::disconnect()
{
    m_dbClient->disconnect();
    LOGI("Connection with DB is closed.");
}

int Storage::ClickHouseClient::getActualVersion() const
{
    return m_dbClient->getActualSchemaVersion();
}

int Storage::ClickHouseClient::getExpectedVersion() const
{
    return m_dbClient->getExpectedSchemaVersion();
}

bool Storage::ClickHouseClient::init()
{
    // Create table "calls"
    db::interfaces::tables::client::create(m_dbClient->db::interfaces::tables::calls::server::tableName,
                                           m_dbClient->db::interfaces::tables::calls::server::tableFields,
                                           m_dbClient->db::interfaces::tables::calls::server::tableSettings);

    // Store current version.
    m_dbClient->putSchemaVersion(getExpectedVersion());

    return true;
}

bool Storage::ClickHouseClient::upgrade(const int actualVersion, const int expectedVersion)
{
    // TODO: implement upgrade functionality.

    // Store current version.
    m_dbClient->putSchemaVersion(expectedVersion);

    return true;
}

void Storage::ClickHouseClient::storeVerdict(const Uvr::CallRecord &record)
{
    db::interfaces::tables::calls::client::put(record);
}

bool Storage::ClickHouseClient::updateVerdict(const Uvr::CallRecord &record)
{
    return db::interfaces::tables::calls::client::update(record);
}
