#pragma once

// Storage includes.
#include "StorageClient.hpp"

// AFAP includes.
#include "components/worker_node/src/config_parameters.hpp"

namespace Storage
{
    namespace API
    {
        template <class TClient>
        bool init(const std::string &configFile);

        /// @brief Registers a @a client for the passed @a uvrId.
        /// @returns @c true if a @a client was registered, @c false if it did not.
        bool registerClient(const Uvr::UvrId_t uvrId,
                            ClientUniquePtr &&client);

        bool start();
        bool isStarted();
        void stop();

        bool storeRecord(const Uvr::UvrId_t uvrId,
                         const Uvr::CallRecord &record);

        size_t updateRecords(const Uvr::UvrId_t uvrId,
                             const Uvr::CallRecords &records);

        void processCallInformationRequest(const Uvr::UvrId_t uvrId,
                                           const Uvr::CallInformationRequest &request);

    } // namespace API

} // namespace Storage

namespace Storage
{
    template <class TClient>
    bool API::init(const std::string &configFile)
    {
        configuration::manager configManager;
        if (configManager.loadConfigFile(configFile) == configuration::errors::FAIL)
        {
            // ERROR is generated inside of called function.
            return false;
        }

        // Create clients.
        // TODO[multi-uvr]: Each UVR must have its own client.
        const auto uvrId = configManager.getValue(UVR_ID, Uvr::UvrId::undefined);
        auto client = std::make_unique<TClient>(uvrId, configManager);
        if (!registerClient(uvrId, std::move(client)))
        {
            // ERROR is generated inside of called function.
            return false;
        }

        return true;
    }

} // namespace Storage
