ROOT  := /usr
SOFT  := ${ROOT}/soft
TOOLS := ${ROOT}/tools
TMP   := ${ROOT}/tmp_`date +"%F-%T"`

.DEFAULT_GOAL := all

all: boost openssl ClickHouse

boost openssl ClickHouse: pre-stage

.PHONY: pre-stage
pre-stage:
	@apt-get update
	@apt-get install -y bash
	@apt-get install -y git
	@apt-get install -y build-essential
	@apt-get install -y wget
	@apt-get install -y python-dev
	@apt-get install -y zip
	@apt-get install -y pkg-config
	@apt-get install -y cmake
	@mkdir -p ${SOFT} && chmod a+w ${SOFT}
	@mkdir -p ${TOOLS} && chmod a+w ${TOOLS}
	-@ln -fs /bin/mkdir /usr/bin/mkdir

###################################
#### BOOST section              ###
###################################
BOOST_VER     := 1.82.0
BOOST_VER_STR := boost_$(subst .,_,${BOOST_VER})

   SEARCH_TO := return std::inserter(target, end(target));
REPLACE_WITH := return std::inserter(target, target.end());
        FILE := ${SOFT}/${BOOST_VER_STR}/boost/json/detail/value_to.hpp

boost_patch = (sed -i "s/${SEARCH_TO}/${REPLACE_WITH}/" ${FILE})

BOOST_SRC     := https://boostorg.jfrog.io/artifactory/main/release/${BOOST_VER}/source/${BOOST_VER_STR}.tar.gz
BOOST_TOOL    := ${TOOLS}/boost

B2_OPTIONS    := -d0 --with-thread --with-json threading=multi link=static

.PHONY: boost
boost:
	@wget -nc ${BOOST_SRC} -P ${SOFT}
	@-rm -r ${SOFT}/${BOOST_VER_STR}
	@tar -xzf ${SOFT}/${BOOST_VER_STR}.tar.gz -C ${SOFT}/
	@$(call boost_patch) #TODO remove after uplift to 1.83
	@bash -xc "pushd ${SOFT}/${BOOST_VER_STR} && \
              ./bootstrap.sh --prefix=${TMP} && \
              ./b2 install ${B2_OPTIONS}     && \
              popd                           && \
              rm -r ${BOOST_TOOL}            ;  \
              mv ${TMP} ${BOOST_TOOL}"
	@-rm -r ${SOFT}/${BOOST_VER_STR}
###################################
#### OPENSSL section            ###
###################################
OPENSSL_VER  := OpenSSL_1_1_1n
OPENSSL_SRC  := git://git.openssl.org/openssl.git --branch ${OPENSSL_VER}
OPENSSL_TOOL := ${TOOLS}/openssl

.PHONY: openssl
openssl:
	@-rm -r ${SOFT}/openssl
	@git clone ${OPENSSL_SRC} ${SOFT}/openssl
	@bash -xc "pushd ${SOFT}/openssl/    && \
               ./config --prefix=${TMP}  && \
               make                      && \
               make install_sw           && \
               popd                      && \
               rm -r ${OPENSSL_TOOL}     ;  \
               mv ${TMP} ${OPENSSL_TOOL}"
	@-rm -r ${SOFT}/openssl
###################################
#### CLICKHOUSE section         ###
###################################
CLCKH_VER=v2.4.0
CLCKH_SRC=https://github.com/ClickHouse/clickhouse-cpp.git --branch ${CLCKH_VER}
CLCKH_SOFT=${SOFT}/clickhouse-cpp
CLCKH_TOOL=${TOOLS}/clickhouse-cpp

.PHONY: ClickHouse
ClickHouse:
	@-rm -r ${CLCKH_SOFT}
	@git clone ${CLCKH_SRC} ${CLCKH_SOFT}
	@bash -xc "pushd ${CLCKH_SOFT}               && \
               mkdir build                       && \
               cd build                          && \
               cmake ..                          && \
               make                              && \
               cmake --install . --prefix ${TMP} && \
               popd                              && \
               rm -r ${CLCKH_TOOL}               ;  \
               mv ${TMP} ${CLCKH_TOOL}"
# workaround to install contrib headers and libraries
	@bash -xc "pushd ${CLCKH_SOFT}/build/contrib                                  && \
               find . -name *.a -exec cp {} ${CLCKH_TOOL}/lib/ \;                 && \
               popd && pushd ${CLCKH_SOFT}/contrib && pushd absl                  && \
               find . -name *.h   -exec cp --parents {} ${CLCKH_TOOL}/include/ \; && \
               find . -name *.inc -exec cp --parents {} ${CLCKH_TOOL}/include/ \; && \
               popd && pushd cityhash                                             && \
               find . -name *.h   -exec cp --parents {} ${CLCKH_TOOL}/include/ \; && \
               popd && pushd lz4                                                  && \
               find . -name *.h   -exec cp --parents {} ${CLCKH_TOOL}/include/ \; && \
               popd && popd"
	@-rm -r ${CLCKH_SOFT}
