override dst ?= /afap
override src ?= ./

UVR_USER   := uvr
UVR_GROUP  := afap
UVR_ROOT   := $(dst)
UVR_BIN    := uvr-sip
CLCKH_USER := uvr

KVR_SRC      := $(src)/release_kvr.zip
UVR_SIP_SRC  := $(src)/release_uvr-sip.zip
SSH_KEY_SRC  := $(src)/id_rsa
SSH_KEY_DST  := $(dst)/id_rsa

HOME    := /home/$(UVR_USER)
TMP     := $(HOME)/tmp
SSH     := $(HOME)/.ssh
SSH_KEY := $(UVR_ROOT)/id_rsa
UVR_BIN := $(UVR_ROOT)/bin

DIGESTS     :=$(UVR_ROOT)/digests
REPORTS     :=$(UVR_ROOT)/reports
CONNECTIONS :=$(UVR_ROOT)/connections

EDIT_JSON   := $(UVR_BIN)/editJson
CFG         := $(UVR_ROOT)/config.json

DIRSTOCREATE := $(HOME) $(TMP) $(UVR_ROOT)
DIRSTOCREATE += $(DIGESTS) $(DIGESTS)/numbers $(DIGESTS)/nodes $(DIGESTS)/operators $(DIGESTS)/pub
DIRSTOCREATE += $(REPORTS)
DIRSTOCREATE += $(CONNECTIONS) $(CONNECTIONS)/requests $(CONNECTIONS)/responses

$(TMP): $(HOME)
$(DIGESTS)/numbers $(DIGESTS)/nodes $(DIGESTS)/operators $(DIGESTS)/pub: $(DIGESTS)
$(CONNECTIONS)/requests $(CONNECTIONS)/responses: $(CONNECTIONS)

$(DIRSTOCREATE): ;
	@-mkdir -p $@

.PHONY: user_add
user_add: $(HOME)
	@-groupadd $(UVR_GROUP)
	@-useradd -M -g $(UVR_GROUP) -s /sbin/nologin $(UVR_USER)


.PHONY: ssh_prepare
ssh_prepare: user_add
	@cp $(SSH_KEY_SRC) $(SSH_KEY_DST)
	@chmod 0600 $(SSH_KEY_DST)
	@chown $(UVR_USER):$(UVR_GROUP) $(SSH_KEY_DST)

.PHONY: pre_build
pre_build:
	@apt update
	@apt install -y zip

.PHONY: build_uvr_sip
build_uvr_sip: _product := uvr-sip
build_uvr_sip:    _root := $(TMP)/release_$(_product)
build_uvr_sip: $(TMP)
	@-rm -rf $(_root)
	@unzip $(UVR_SIP_SRC) -d $(TMP)
	@$(MAKE) build   product=$(_product) -C $(_root)
	@$(MAKE) install product=$(_product) -C $(_root) dest=$(UVR_ROOT)
	@-rm -rf $(_root)

EMPTY := ""

.PHONY: cfg_create
cfg_create:
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/vfgw/http/privateKeyPath" "\"$(EMPTY)\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/ccn/sftp private key" "\"$(SSH_KEY_DST)\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/ccngw/digests/numbers" "\"$(DIGESTS)/numbers\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/ccngw/digests/nodes" "\"$(DIGESTS)/nodes\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/ccngw/digests/operators" "\"$(DIGESTS)/operators\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/ccngw/digests/pub" "\"$(DIGESTS)/pub\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/ccngw/connections/requests" "\"$(CONNECTIONS)/requests\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/ccngw/connections/responses" "\"$(CONNECTIONS)/responses\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/ccngw/reports/incidents" "\"$(REPORTS)/incidents\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/ccngw/reports/incidents_a" "\"$(REPORTS)/incidents_a\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/ccngw/reports/statistics" "\"$(REPORTS)/statistics\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/sip/incomingCallsPort" "\"7321\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/sip/outgoingCallsPort" "\"7322\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/sip/interface" "\"eth0\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/storage/clckh/database" "\"$(CLCKH_USER)\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/storage/clckh/user" "\"$(CLCKH_USER)\""
	@$(EDIT_JSON) $(CFG) $(CFG) replace "/storage/clckh/password" "\"$(CLCKH_USER)\""

.PHONY: build
build: build_uvr_sip

.PHONY: pre_install
pre_install:
	@apt update
	@apt install -y sudo
	@apt install -y openssh-client

.PHONY: install
install: pre_install user_add ssh_prepare cfg_create $(DIRSTOCREATE)
	@chown -R $(UVR_USER):$(UVR_GROUP) $(UVR_ROOT)
	@$(foreach file, $(wildcard $(UVR_ROOT)/deploy/*.sh), chmod a+x $(file) ;)
	@ln -sf $(UVR_ROOT)/deploy/uvr.service /etc/systemd/system/uvr.service
	@sed -i 's:<FIXME>:$(UVR_ROOT):' $(UVR_ROOT)/deploy/uvr.sh
	@ln -sf $(UVR_ROOT)/deploy/uvr.sh /etc/systemd/system/uvr.sh
	@-systemctl daemon-reload

MAKEFILE_COMPLETE := $(CURDIR)/$(firstword $(MAKEFILE_LIST))

.PHONY: deploy
deploy: build
	@$(MAKE) -f $(MAKEFILE_COMPLETE) install

.PHONY: help
help:
	@printf "Usage: make -f <path_to_makefile> src=<src> dst=<dst> <target>\n"
	@printf "\t<src> - path to a folder containing:\n\t\t - id_rsa\n\t\t - release_kvr.zip\n\t\t - release_uvr-sip.zip\n"
	@printf "\t<dst> - path to a folder UVR to be deployed\n"
