#include "UvrAPI.hpp"
#include "modules/sip/src/node.hpp"
#include "modules/storage/src/StorageApi.hpp"
#include "modules/storage/src/ClickHouseClient.hpp"

namespace
{
    void signalHandler(int sigNo)
    {
        LOGD("Received signal: ", sigNo);
        Uvr::API::stop();
    }
} // namespace

int main(int argc, char **argv)
{
    // Register to handle the signals that indicate when the server should exit.
    std::signal(SIGINT, signalHandler);
    std::signal(SIGTERM, signalHandler);
#ifdef SIGQUIT
    std::signal(SIGQUIT, signalHandler);
#endif // ifdef SIGQUIT

    if (argc < 2)
    {
        LOGERROR("No configuration file is provided");
        return 1;
    }

    std::string param1(argv[1]);

    if (param1 == "-v")
    {
        std::cout << Uvr::Troubleshoot::doVersions();
        return 0;
    }

    std::string confname(param1);

    // Initialize Verification Component
    if(!Uvr::API::init(confname))
    {
        LOGE("There are problems with \"", confname, "\" configuration file. UVR can't start!");
        return 1;
    }

    // Initialize Storage Module with ClickHouse DB
    Storage::API::init<Storage::ClickHouseClient>(confname);

    // Register call information callback to get info from ClickHouse DB
    Uvr::UserId_t apiUser =
        Uvr::API::registerCallInformationRequestCallback(Storage::API::processCallInformationRequest);

    // Register store record callback.
    Uvr::API::registerStoreRecordCallback(Storage::API::storeRecord, apiUser);

    // Initialize SIP Module
    modules::sip::api::init(param1, apiUser);

    // Register update database callback for CLI.
    Uvr::API::registerUpdateRecordsCallback(Storage::API::updateRecords, apiUser);

    // Start Verification Component
    Uvr::API::start();
    // Start Storage Module
    Storage::API::start();
    // Start SIP Module
    modules::sip::api::start();

    // Go to Main loop
    Uvr::API::waitStop();

    modules::sip::api::stop();
    modules::sip::api::wait_stop();
}
